@@ -3,8 +3,8 @@
 #include <stdlib.h>
 #include <libecal/e-cal.h>
 
-int
-main (int argc, char **argv)
+gint
+main (gint argc, gchar **argv)
 {
 	ECal *ecal;
 	GList *l, *objects;
@@ -20,18 +20,18 @@ main (int argc, char **argv)
 
 	if (!e_cal_open (ecal, TRUE, NULL)) {
 		printf ("failed to open calendar\n");
-		exit(0);
+		exit (0);
 	}
 
 	if (!e_cal_get_object_list_as_comp (ecal, argv[2], &objects, NULL)) {
 		printf ("failed to get objects\n");
-		exit(0);
+		exit (0);
 	}
 
 	printf ("Received %d objects\n", g_list_length (objects));
 	for (l = objects; l; l = l->next) {
 		ECalComponent *comp = E_CAL_COMPONENT (l->data);
-		char *str;
+		gchar *str;
 
 		str = e_cal_component_get_as_string (comp);
 		printf ("%s\n", str);
