@@ -4,8 +4,8 @@
  *
  * Copyright (C) 1999-2008 Novell, Inc. (www.novell.com)
  *
- * This program is free software; you can redistribute it and/or 
- * modify it under the terms of version 2 of the GNU Lesser General Public 
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of version 2 of the GNU Lesser General Public
  * License as published by the Free Software Foundation.
  *
  * This program is distributed in the hope that it will be useful,
@@ -19,27 +19,21 @@
  * USA
  */
 
-#include <pthread.h>
 #include <stdio.h>
 #include <stdlib.h>
 #include <string.h>
 
-#include <libedataserver/e-data-server-util.h>
-
 #include "camel-debug.h"
 
-int camel_verbose_debug;
+gint camel_verbose_debug;
 
 static GHashTable *debug_table = NULL;
 
 /**
  * camel_debug_init:
- * @void: 
- * 
- * Init camel debug.  Maintain legacy CAMEL_VERBOSE_DEBUG as well as the
- * new CAMEL_DEBUG based environment variable interfaces.
+ * @void:
  *
- * CAMEL_VERBOSE_DEBUG is set to a number to turn debug on.
+ * Init camel debug.
  *
  * CAMEL_DEBUG is set to a comma separated list of modules to debug.
  * The modules can contain module-specific specifiers after a ':', or
@@ -47,26 +41,23 @@ static GHashTable *debug_table = NULL;
  * for imap debug, or 'imap:folder' for imap folder debug.  Additionaly,
  * ':folder' can be used for a wildcard for any folder operations.
  **/
-void camel_debug_init(void)
+void
+camel_debug_init (void)
 {
-	char *d;
-
-	d = getenv("CAMEL_VERBOSE_DEBUG");
-	if (d)
-		camel_verbose_debug = atoi(d);
+	gchar *d;
 
 	d = g_strdup(getenv("CAMEL_DEBUG"));
 	if (d) {
-		char *p;
+		gchar *p;
 
-		debug_table = g_hash_table_new(g_str_hash, g_str_equal);
+		debug_table = g_hash_table_new (g_str_hash, g_str_equal);
 		p = d;
 		while (*p) {
 			while (*p && *p != ',')
 				p++;
 			if (*p)
 				*p++ = 0;
-			g_hash_table_insert(debug_table, d, d);
+			g_hash_table_insert (debug_table, d, d);
 			d = p;
 		}
 
@@ -77,64 +68,64 @@ void camel_debug_init(void)
 
 /**
  * camel_debug:
- * @mode: 
- * 
+ * @mode:
+ *
  * Check to see if a debug mode is activated.  @mode takes one of two forms,
  * a fully qualified 'module:target', or a wildcard 'module' name.  It
  * returns a boolean to indicate if the module or module and target is
  * currently activated for debug output.
- * 
- * Return value: 
+ *
+ * Returns:
  **/
-gboolean camel_debug(const char *mode)
+gboolean camel_debug (const gchar *mode)
 {
 	if (camel_verbose_debug)
 		return TRUE;
 
 	if (debug_table) {
-		char *colon;
-		char *fallback;
+		gchar *colon;
+		gchar *fallback;
 
-		if (g_hash_table_lookup(debug_table, mode))
+		if (g_hash_table_lookup (debug_table, mode))
 			return TRUE;
 
 		/* Check for fully qualified debug */
-		colon = strchr(mode, ':');
+		colon = strchr (mode, ':');
 		if (colon) {
-			fallback = g_alloca(strlen(mode)+1);
-			strcpy(fallback, mode);
+			fallback = g_alloca (strlen (mode)+1);
+			strcpy (fallback, mode);
 			colon = (colon-mode) + fallback;
 			/* Now check 'module[:*]' */
 			*colon = 0;
-			if (g_hash_table_lookup(debug_table, fallback))
+			if (g_hash_table_lookup (debug_table, fallback))
 				return TRUE;
 			/* Now check ':subsystem' */
 			*colon = ':';
-			if (g_hash_table_lookup(debug_table, colon))
-				return TRUE;		
+			if (g_hash_table_lookup (debug_table, colon))
+				return TRUE;
 		}
 	}
 
 	return FALSE;
 }
 
-static pthread_mutex_t debug_lock = PTHREAD_MUTEX_INITIALIZER;
+static GStaticMutex debug_lock = G_STATIC_MUTEX_INIT;
 /**
  * camel_debug_start:
  * @mode:
- * 
+ *
  * Start debug output for a given mode, used to make sure debug output
  * is output atomically and not interspersed with unrelated stuff.
  *
- * Return value: Returns true if mode is set, and in which case, you must
+ * Returns: Returns true if mode is set, and in which case, you must
  * call debug_end when finished any screen output.
  **/
 gboolean
-camel_debug_start(const char *mode)
+camel_debug_start (const gchar *mode)
 {
-	if (camel_debug(mode)) {
-		pthread_mutex_lock(&debug_lock);
-		printf("Thread %" G_GINT64_MODIFIER "x >\n", e_util_pthread_id(pthread_self()));
+	if (camel_debug (mode)) {
+		g_static_mutex_lock (&debug_lock);
+		printf ("Thread %p >\n", g_thread_self());
 		return TRUE;
 	}
 
@@ -143,22 +134,22 @@ camel_debug_start(const char *mode)
 
 /**
  * camel_debug_end:
- * 
+ *
  * Call this when you're done with your debug output.  If and only if
  * you called camel_debug_start, and if it returns TRUE.
  **/
 void
-camel_debug_end(void)
+camel_debug_end (void)
 {
-	printf("< %" G_GINT64_MODIFIER "x >\n", e_util_pthread_id(pthread_self()));
-	pthread_mutex_unlock(&debug_lock);
+	printf ("< %p >\n", g_thread_self());
+	g_static_mutex_unlock (&debug_lock);
 }
 
 #if 0
 #include <sys/debugreg.h>
 
 static unsigned
-i386_length_and_rw_bits (int len, enum target_hw_bp_type type)
+i386_length_and_rw_bits (gint len, enum target_hw_bp_type type)
 {
   unsigned rw;
 
@@ -181,7 +172,7 @@ i386_length_and_rw_bits (int len, enum target_hw_bp_type type)
 #endif
       default:
         internal_error (__FILE__, __LINE__, "\
-Invalid hw breakpoint type %d in i386_length_and_rw_bits.\n", (int)type);
+Invalid hw breakpoint type %d in i386_length_and_rw_bits.\n", (gint)type);
     }
 
   switch (len)
@@ -210,7 +201,6 @@ Invalid hw breakpoint length %d in i386_length_and_rw_bits.\n", len);
 #define I386_DR_LOCAL_ENABLE(i) \
   dr_control_mirror |= (1 << (DR_LOCAL_ENABLE_SHIFT + DR_ENABLE_SIZE * (i)))
 
-
 #define set_dr(regnum, val) \
 		__asm__("movl %0,%%db" #regnum  \
 			: /* no output */ \
@@ -222,17 +212,17 @@ Invalid hw breakpoint length %d in i386_length_and_rw_bits.\n", len);
 
 /* fine idea, but it doesn't work, crashes in get_dr :-/ */
 void
-camel_debug_hwatch(int wp, void *addr)
+camel_debug_hwatch (gint wp, gpointer addr)
 {
      guint32 control, rw;
 
-     g_assert(wp <= DR_LASTADDR);
-     g_assert(sizeof(addr) == 4);
+     g_assert (wp <= DR_LASTADDR);
+     g_assert (sizeof (addr) == 4);
 
-     get_dr(7, control);
+     get_dr (7, control);
      /* set watch mode + size */
      rw = DR_RW_WRITE | DR_LEN_4;
-     control &= ~(((1<<DR_CONTROL_SIZE)-1) << (DR_CONTROL_SHIFT+DR_CONTROL_SIZE * wp));
+     control &= ~(((1 << DR_CONTROL_SIZE)-1) << (DR_CONTROL_SHIFT+DR_CONTROL_SIZE * wp));
      control |= rw << (DR_CONTROL_SHIFT + DR_CONTROL_SIZE*wp);
      /* set watch enable */
      control |=  ( 1<< (DR_LOCAL_ENABLE_SHIFT + DR_ENABLE_SIZE * wp));
@@ -241,19 +231,19 @@ camel_debug_hwatch(int wp, void *addr)
 
      switch (wp) {
      case 0:
-	     set_dr(0, addr);
+	     set_dr (0, addr);
 	     break;
      case 1:
-	     set_dr(1, addr);
+	     set_dr (1, addr);
 	     break;
      case 2:
-	     set_dr(2, addr);
+	     set_dr (2, addr);
 	     break;
      case 3:
-	     set_dr(3, addr);
+	     set_dr (3, addr);
 	     break;
      }
-     set_dr(7, control);
+     set_dr (7, control);
 }
 
 #endif
