@@ -53,7 +53,7 @@
 /**
  * CAMEL_FOLDER_ERROR:
  *
- * Since: 3.0
+ * Since: 2.32
  **/
 #define CAMEL_FOLDER_ERROR \
 	(camel_folder_error_quark ())
@@ -72,7 +72,7 @@ typedef struct _CamelFolderPrivate CamelFolderPrivate;
 /**
  * CamelFolderError:
  *
- * Since: 3.0
+ * Since: 2.32
  **/
 typedef enum {
 	CAMEL_FOLDER_ERROR_INVALID,
@@ -88,7 +88,7 @@ typedef enum {
 /**
  * CamelFolderLock:
  *
- * Since: 3.0
+ * Since: 2.32
  **/
 typedef enum {
 	CAMEL_FOLDER_CHANGE_LOCK,
@@ -119,50 +119,43 @@ struct _CamelFolderQuotaInfo {
 	struct _CamelFolderQuotaInfo *next;
 };
 
+typedef enum {
+	CAMEL_FOLDER_HAS_SUMMARY_CAPABILITY = 1 << 0,
+	CAMEL_FOLDER_HAS_SEARCH_CAPABILITY  = 1 << 1,
+	CAMEL_FOLDER_FILTER_RECENT          = 1 << 2,
+	CAMEL_FOLDER_HAS_BEEN_DELETED       = 1 << 3,
+	CAMEL_FOLDER_IS_TRASH               = 1 << 4,
+	CAMEL_FOLDER_IS_JUNK                = 1 << 5,
+	CAMEL_FOLDER_FILTER_JUNK            = 1 << 6
+} CamelFolderFlags;
+
 struct _CamelFolder {
 	CamelObject parent;
 	CamelFolderPrivate *priv;
 
 	CamelFolderSummary *summary;
 
-	guint32 folder_flags;
-	guint32 permanent_flags;
+	CamelFolderFlags folder_flags;
+	CamelMessageFlags permanent_flags;
 
 	/* Future ABI expansion */
 	gpointer later[4];
 };
 
-#define CAMEL_FOLDER_HAS_SUMMARY_CAPABILITY (1<<0)
-#define CAMEL_FOLDER_HAS_SEARCH_CAPABILITY  (1<<1)
-#define CAMEL_FOLDER_FILTER_RECENT          (1<<2)
-#define CAMEL_FOLDER_HAS_BEEN_DELETED       (1<<3)
-#define CAMEL_FOLDER_IS_TRASH               (1<<4)
-#define CAMEL_FOLDER_IS_JUNK                (1<<5)
-#define CAMEL_FOLDER_FILTER_JUNK	    (1<<6)
-
 struct _CamelFolderClass {
 	CamelObjectClass parent_class;
 
-	/* Methods */
-	gboolean	(*refresh_info)		(CamelFolder *folder,
-						 GError **error);
-	gboolean	(*sync)			(CamelFolder *folder,
-						 gboolean expunge,
-						 GError **error);
-	gboolean	(*expunge)		(CamelFolder *folder,
-						 GError **error);
+	/* Non-Blocking Methods */
 	gint		(*get_message_count)	(CamelFolder *folder);
-	gboolean	(*append_message)	(CamelFolder *folder,
-						 CamelMimeMessage *message,
-						 const CamelMessageInfo *info,
-						 gchar **appended_uid,
-						 GError **error);
-	guint32		(*get_permanent_flags)	(CamelFolder *folder);
-	guint32		(*get_message_flags)	(CamelFolder *folder,
+	CamelMessageFlags
+			(*get_permanent_flags)	(CamelFolder *folder);
+	CamelMessageFlags
+			(*get_message_flags)	(CamelFolder *folder,
 						 const gchar *uid);
 	gboolean	(*set_message_flags)	(CamelFolder *folder,
 						 const gchar *uid,
-						 guint32 flags, guint32 set);
+						 CamelMessageFlags flags,
+						 CamelMessageFlags set);
 	gboolean	(*get_message_user_flag)(CamelFolder *folder,
 						 const gchar *uid,
 						 const gchar *name);
@@ -177,10 +170,6 @@ struct _CamelFolderClass {
 						 const gchar *uid,
 						 const gchar *name,
 						 const gchar *value);
-	CamelMimeMessage *
-			(*get_message)		(CamelFolder *folder,
-						 const gchar *uid,
-						 GError **error);
 	GPtrArray *	(*get_uids)		(CamelFolder *folder);
 	void		(*free_uids)		(CamelFolder *folder,
 						 GPtrArray *array);
@@ -209,12 +198,6 @@ struct _CamelFolderClass {
 						 CamelMessageInfo *info);
 	void		(*free_message_info)	(CamelFolder *folder,
 						 CamelMessageInfo *info);
-	gboolean	(*transfer_messages_to)	(CamelFolder *source,
-						 GPtrArray *uids,
-						 CamelFolder *destination,
-						 GPtrArray **transferred_uids,
-						 gboolean delete_originals,
-						 GError **error);
 	void		(*delete)		(CamelFolder *folder);
 	void		(*rename)		(CamelFolder *folder,
 						 const gchar *newname);
@@ -226,9 +209,6 @@ struct _CamelFolderClass {
 	guint32		(*count_by_expression)	(CamelFolder *folder,
 						 const gchar *expression,
 						 GError **error);
-	gboolean	(*sync_message)		(CamelFolder *folder,
-						 const gchar *uid,
-						 GError **error);
 	GPtrArray *	(*get_uncached_uids)	(CamelFolder *folder,
 						 GPtrArray *uids,
 						 GError **error);
@@ -236,6 +216,114 @@ struct _CamelFolderClass {
 						 const gchar *uid,
 						 GError **error);
 
+	/* Synchronous I/O Methods */
+	gboolean	(*append_message_sync)	(CamelFolder *folder,
+						 CamelMimeMessage *message,
+						 CamelMessageInfo *info,
+						 gchar **appended_uid,
+						 GCancellable *cancellable,
+						 GError **error);
+	gboolean	(*expunge_sync)		(CamelFolder *folder,
+						 GCancellable *cancellable,
+						 GError **error);
+	CamelMimeMessage *
+			(*get_message_sync)	(CamelFolder *folder,
+						 const gchar *message_uid,
+						 GCancellable *cancellable,
+						 GError **error);
+	gboolean	(*refresh_info_sync)	(CamelFolder *folder,
+						 GCancellable *cancellable,
+						 GError **error);
+	gboolean	(*synchronize_sync)	(CamelFolder *folder,
+						 gboolean expunge,
+						 GCancellable *cancellable,
+						 GError **error);
+	gboolean	(*synchronize_message_sync)
+						(CamelFolder *folder,
+						 const gchar *message_uid,
+						 GCancellable *cancellable,
+						 GError **error);
+	gboolean	(*transfer_messages_to_sync)
+						(CamelFolder *source,
+						 GPtrArray *message_uids,
+						 CamelFolder *destination,
+						 gboolean delete_originals,
+						 GPtrArray **transferred_uids,
+						 GCancellable *cancellable,
+						 GError **error);
+
+	/* Asynchronous I/O Methods (all have defaults) */
+	void		(*append_message)	(CamelFolder *folder,
+						 CamelMimeMessage *message,
+						 CamelMessageInfo *info,
+						 gint io_priority,
+						 GCancellable *cancellable,
+						 GAsyncReadyCallback callback,
+						 gpointer user_data);
+	gboolean	(*append_message_finish)
+						(CamelFolder *folder,
+						 GAsyncResult *result,
+						 gchar **appended_uid,
+						 GError **error);
+	void		(*expunge)		(CamelFolder *folder,
+						 gint io_priority,
+						 GCancellable *cancellable,
+						 GAsyncReadyCallback callback,
+						 gpointer user_data);
+	gboolean	(*expunge_finish)	(CamelFolder *folder,
+						 GAsyncResult *result,
+						 GError **error);
+	void		(*get_message)		(CamelFolder *folder,
+						 const gchar *message_uid,
+						 gint io_priority,
+						 GCancellable *cancellable,
+						 GAsyncReadyCallback callback,
+						 gpointer user_data);
+	CamelMimeMessage *
+			(*get_message_finish)	(CamelFolder *folder,
+						 GAsyncResult *result,
+						 GError **error);
+	void		(*refresh_info)		(CamelFolder *folder,
+						 gint io_priority,
+						 GCancellable *cancellable,
+						 GAsyncReadyCallback callback,
+						 gpointer user_data);
+	gboolean	(*refresh_info_finish)	(CamelFolder *folder,
+						 GAsyncResult *result,
+						 GError **error);
+	void		(*synchronize)		(CamelFolder *folder,
+						 gboolean expunge,
+						 gint io_priority,
+						 GCancellable *cancellable,
+						 GAsyncReadyCallback callback,
+						 gpointer user_data);
+	gboolean	(*synchronize_finish)	(CamelFolder *folder,
+						 GAsyncResult *result,
+						 GError **error);
+	void		(*synchronize_message)	(CamelFolder *folder,
+						 const gchar *message_uid,
+						 gint io_priority,
+						 GCancellable *cancellable,
+						 GAsyncReadyCallback callback,
+						 gpointer user_data);
+	gboolean	(*synchronize_message_finish)
+						(CamelFolder *folder,
+						 GAsyncResult *result,
+						 GError **error);
+	void		(*transfer_messages_to)	(CamelFolder *source,
+						 GPtrArray *message_uids,
+						 CamelFolder *destination,
+						 gboolean delete_originals,
+						 gint io_priority,
+						 GCancellable *cancellable,
+						 GAsyncReadyCallback callback,
+						 gpointer user_data);
+	gboolean	(*transfer_messages_to_finish)
+						(CamelFolder *source,
+						 GAsyncResult *result,
+						 GPtrArray **transferred_uids,
+						 GError **error);
+
 	/* Signals */
 	void		(*changed)		(CamelFolder *folder,
 						 CamelFolderChangeInfo *info);
@@ -246,22 +334,10 @@ struct _CamelFolderClass {
 
 GType		camel_folder_get_type		(void);
 GQuark		camel_folder_error_quark	(void) G_GNUC_CONST;
-gboolean	camel_folder_refresh_info	(CamelFolder *folder,
-						 GError **error);
-gboolean	camel_folder_sync		(CamelFolder *folder,
-						 gboolean expunge,
-						 GError **error);
 void		camel_folder_set_lock_async	(CamelFolder *folder,
 						 gboolean skip_folder_lock);
-
 struct _CamelStore *
 		camel_folder_get_parent_store	(CamelFolder *folder);
-
-/* delete operations */
-gboolean	camel_folder_expunge		(CamelFolder *folder,
-						 GError **error);
-
-/* folder name operations */
 const gchar *	camel_folder_get_name		(CamelFolder *folder);
 void		camel_folder_set_name		(CamelFolder *folder,
 						 const gchar *name);
@@ -271,80 +347,55 @@ void		camel_folder_set_full_name	(CamelFolder *folder,
 const gchar *	camel_folder_get_description	(CamelFolder *folder);
 void		camel_folder_set_description	(CamelFolder *folder,
 						 const gchar *description);
-
-/* various properties accessors */
-guint32		camel_folder_get_permanent_flags(CamelFolder *folder);
-
+const gchar *	camel_folder_get_uri		(CamelFolder *folder);
+CamelMessageFlags
+		camel_folder_get_permanent_flags
+						(CamelFolder *folder);
 #ifndef CAMEL_DISABLE_DEPRECATED
-guint32		camel_folder_get_message_flags	(CamelFolder *folder,
+CamelMessageFlags
+		camel_folder_get_message_flags	(CamelFolder *folder,
 						 const gchar *uid);
-
 gboolean	camel_folder_set_message_flags	(CamelFolder *folder,
 						 const gchar *uid,
-						 guint32 flags,
-						 guint32 set);
-
+						 CamelMessageFlags flags,
+						 CamelMessageFlags set);
 gboolean	camel_folder_get_message_user_flag
 						(CamelFolder *folder,
 						 const gchar *uid,
 						 const gchar *name);
-
 void		camel_folder_set_message_user_flag
 						(CamelFolder *folder,
 						 const gchar *uid,
 						 const gchar *name,
 						 gboolean value);
-
 const gchar *	camel_folder_get_message_user_tag
 						(CamelFolder *folder,
 						 const gchar *uid,
 						 const gchar *name);
-
 void		camel_folder_set_message_user_tag
 						(CamelFolder *folder,
 						 const gchar *uid,
 						 const gchar *name,
 						 const gchar *value);
 #endif /* CAMEL_DISABLE_DEPRECATED */
-
-/* message manipulation */
-gboolean	camel_folder_append_message	(CamelFolder *folder,
-						 CamelMimeMessage *message,
-						 const CamelMessageInfo *info,
-						 gchar **appended_uid,
-						 GError **error);
-
-/* summary related operations */
 gboolean	camel_folder_has_summary_capability
 						(CamelFolder *folder);
-
 gint		camel_folder_get_message_count	(CamelFolder *folder);
-
 #ifndef CAMEL_DISABLE_DEPRECATED
 gint		camel_folder_get_unread_message_count
 						(CamelFolder *folder);
 #endif
-
 gint		camel_folder_get_deleted_message_count
 						(CamelFolder *folder);
-
 GPtrArray *	camel_folder_get_summary	(CamelFolder *folder);
 void		camel_folder_free_summary	(CamelFolder *folder,
 						 GPtrArray *array);
 
-/* uid based access operations */
-CamelMimeMessage *
-		camel_folder_get_message	(CamelFolder *folder,
-						 const gchar *uid,
-						 GError **error);
-gboolean	camel_folder_sync_message	(CamelFolder *folder,
-						 const gchar *uid,
-						 GError **error);
-
 #define camel_folder_delete_message(folder, uid) \
-	(camel_folder_set_message_flags \
-	(folder, uid, CAMEL_MESSAGE_DELETED | \
-	CAMEL_MESSAGE_SEEN, CAMEL_MESSAGE_DELETED|CAMEL_MESSAGE_SEEN))
+	(camel_folder_set_message_flags ( \
+		folder, uid, \
+		CAMEL_MESSAGE_DELETED | CAMEL_MESSAGE_SEEN, \
+		CAMEL_MESSAGE_DELETED | CAMEL_MESSAGE_SEEN))
 
 GPtrArray *	camel_folder_get_uids		(CamelFolder *folder);
 void		camel_folder_free_uids		(CamelFolder *folder,
@@ -357,8 +408,6 @@ gint		camel_folder_cmp_uids		(CamelFolder *folder,
 						 const gchar *uid2);
 void		camel_folder_sort_uids		(CamelFolder *folder,
 						 GPtrArray *uids);
-
-/* search api */
 gboolean	camel_folder_has_search_capability
 						(CamelFolder *folder);
 GPtrArray *	camel_folder_search_by_expression
@@ -371,11 +420,9 @@ GPtrArray *	camel_folder_search_by_uids	(CamelFolder *folder,
 						 GError **error);
 void		camel_folder_search_free	(CamelFolder *folder,
 						 GPtrArray *result);
-guint32		camel_folder_count_by_expression(CamelFolder *folder,
+guint32		camel_folder_count_by_expression (CamelFolder *folder,
 						 const gchar *expression,
 						 GError **error);
-
-/* summary info */
 CamelMessageInfo *
 		camel_folder_get_message_info	(CamelFolder *folder,
 						 const gchar *uid);
@@ -385,28 +432,15 @@ void		camel_folder_free_message_info	(CamelFolder *folder,
 void		camel_folder_ref_message_info	(CamelFolder *folder,
 						 CamelMessageInfo *info);
 #endif
-
-gboolean	camel_folder_transfer_messages_to
-						(CamelFolder *source,
-						 GPtrArray *uids,
-						 CamelFolder *dest,
-						 GPtrArray **transferred_uids,
-						 gboolean delete_originals,
-						 GError **error);
-
 void		camel_folder_delete		(CamelFolder *folder);
 void		camel_folder_rename		(CamelFolder *folder,
-						 const gchar *new);
+						 const gchar *new_name);
 void		camel_folder_changed		(CamelFolder *folder,
 						 CamelFolderChangeInfo *changes);
-
-/* stop/restart getting events */
 void		camel_folder_freeze		(CamelFolder *folder);
 void		camel_folder_thaw		(CamelFolder *folder);
 gboolean	camel_folder_is_frozen		(CamelFolder *folder);
 gint		camel_folder_get_frozen_count	(CamelFolder *folder);
-
-/* quota support */
 CamelFolderQuotaInfo *
 		camel_folder_get_quota_info	(CamelFolder *folder);
 CamelFolderQuotaInfo *
@@ -416,25 +450,135 @@ CamelFolderQuotaInfo *
 CamelFolderQuotaInfo *
 		camel_folder_quota_info_clone	(const CamelFolderQuotaInfo *info);
 void		camel_folder_quota_info_free	(CamelFolderQuotaInfo *info);
-
-/* For use by subclasses (for free_{uids,summary,subfolder_names}) */
 void		camel_folder_free_nop		(CamelFolder *folder,
 						 GPtrArray *array);
 void		camel_folder_free_shallow	(CamelFolder *folder,
 						 GPtrArray *array);
 void		camel_folder_free_deep		(CamelFolder *folder,
 						 GPtrArray *array);
-
 gchar *		camel_folder_get_filename	(CamelFolder *folder,
 						 const gchar *uid,
 						 GError **error);
+void		camel_folder_lock		(CamelFolder *folder,
+						 CamelFolderLock lock);
+void		camel_folder_unlock		(CamelFolder *folder,
+						 CamelFolderLock lock);
+
+gboolean	camel_folder_append_message_sync
+						(CamelFolder *folder,
+						 CamelMimeMessage *message,
+						 CamelMessageInfo *info,
+						 gchar **appended_uid,
+						 GCancellable *cancellable,
+						 GError **error);
+void		camel_folder_append_message	(CamelFolder *folder,
+						 CamelMimeMessage *message,
+						 CamelMessageInfo *info,
+						 gint io_priority,
+						 GCancellable *cancellable,
+						 GAsyncReadyCallback callback,
+						 gpointer user_data);
+gboolean	camel_folder_append_message_finish
+						(CamelFolder *folder,
+						 GAsyncResult *result,
+						 gchar **appended_uid,
+						 GError **error);
+gboolean	camel_folder_expunge_sync	(CamelFolder *folder,
+						 GCancellable *cancellable,
+						 GError **error);
+void		camel_folder_expunge		(CamelFolder *folder,
+						 gint io_priority,
+						 GCancellable *cancellable,
+						 GAsyncReadyCallback callback,
+						 gpointer user_data);
+gboolean	camel_folder_expunge_finish	(CamelFolder *folder,
+						 GAsyncResult *result,
+						 GError **error);
+CamelMimeMessage *
+		camel_folder_get_message_sync	(CamelFolder *folder,
+						 const gchar *message_uid,
+						 GCancellable *cancellable,
+						 GError **error);
+void		camel_folder_get_message	(CamelFolder *folder,
+						 const gchar *message_uid,
+						 gint io_priority,
+						 GCancellable *cancellable,
+						 GAsyncReadyCallback callback,
+						 gpointer user_data);
+CamelMimeMessage *
+		camel_folder_get_message_finish	(CamelFolder *folder,
+						 GAsyncResult *result,
+						 GError **error);
+gboolean	camel_folder_refresh_info_sync	(CamelFolder *folder,
+						 GCancellable *cancellable,
+						 GError **error);
+void		camel_folder_refresh_info	(CamelFolder *folder,
+						 gint io_priority,
+						 GCancellable *cancellable,
+						 GAsyncReadyCallback callback,
+						 gpointer user_data);
+gboolean	camel_folder_refresh_info_finish
+						(CamelFolder *folder,
+						 GAsyncResult *result,
+						 GError **error);
+gboolean	camel_folder_synchronize_sync	(CamelFolder *folder,
+						 gboolean expunge,
+						 GCancellable *cancellable,
+						 GError **error);
+void		camel_folder_synchronize	(CamelFolder *folder,
+						 gboolean expunge,
+						 gint io_priority,
+						 GCancellable *cancellable,
+						 GAsyncReadyCallback callback,
+						 gpointer user_data);
+gboolean	camel_folder_synchronize_finish	(CamelFolder *folder,
+						 GAsyncResult *result,
+						 GError **error);
+gboolean	camel_folder_synchronize_message_sync
+						(CamelFolder *folder,
+						 const gchar *message_uid,
+						 GCancellable *cancellable,
+						 GError **error);
+void		camel_folder_synchronize_message
+						(CamelFolder *folder,
+						 const gchar *message_uid,
+						 gint io_priority,
+						 GCancellable *cancellable,
+						 GAsyncReadyCallback callback,
+						 gpointer user_data);
+gboolean	camel_folder_synchronize_message_finish
+						(CamelFolder *folder,
+						 GAsyncResult *result,
+						 GError **error);
+gboolean	camel_folder_transfer_messages_to_sync
+						(CamelFolder *source,
+						 GPtrArray *message_uids,
+						 CamelFolder *destination,
+						 gboolean delete_originals,
+						 GPtrArray **transferred_uids,
+						 GCancellable *cancellable,
+						 GError **error);
+void		camel_folder_transfer_messages_to
+						(CamelFolder *source,
+						 GPtrArray *message_uids,
+						 CamelFolder *destination,
+						 gboolean delete_originals,
+						 gint io_priority,
+						 GCancellable *cancellable,
+						 GAsyncReadyCallback callback,
+						 gpointer user_data);
+gboolean	camel_folder_transfer_messages_to_finish
+						(CamelFolder *source,
+						 GAsyncResult *result,
+						 GPtrArray **transferred_uids,
+						 GError **error);
 
 /* update functions for change info */
 CamelFolderChangeInfo *
 		camel_folder_change_info_new	(void);
 void		camel_folder_change_info_clear	(CamelFolderChangeInfo *info);
 void		camel_folder_change_info_free	(CamelFolderChangeInfo *info);
-gboolean	camel_folder_change_info_changed(CamelFolderChangeInfo *info);
+gboolean	camel_folder_change_info_changed (CamelFolderChangeInfo *info);
 
 /* for building diff's automatically */
 void		camel_folder_change_info_add_source
@@ -455,7 +599,7 @@ void		camel_folder_change_info_build_diff
 /* for manipulating diff's directly */
 void		camel_folder_change_info_cat	(CamelFolderChangeInfo *info,
 						 CamelFolderChangeInfo *src);
-void		camel_folder_change_info_add_uid(CamelFolderChangeInfo *info,
+void		camel_folder_change_info_add_uid (CamelFolderChangeInfo *info,
 						 const gchar *uid);
 void		camel_folder_change_info_remove_uid
 						(CamelFolderChangeInfo *info,
@@ -466,10 +610,6 @@ void		camel_folder_change_info_change_uid
 void		camel_folder_change_info_recent_uid
 						(CamelFolderChangeInfo *info,
 						 const gchar *uid);
-void		camel_folder_lock		(CamelFolder *folder,
-						 CamelFolderLock lock);
-void		camel_folder_unlock		(CamelFolder *folder,
-						 CamelFolderLock lock);
 
 G_END_DECLS
 
