@@ -34,8 +34,6 @@
 
 #include <glib/gi18n-lib.h>
 
-#include <libedataserver/e-data-server-util.h>
-
 #include "camel-debug.h"
 #include "camel-operation.h"
 #include "camel-service.h"
@@ -61,7 +59,8 @@ service_finalize (GObject *object)
 	CamelService *service = CAMEL_SERVICE (object);
 
 	if (service->status == CAMEL_SERVICE_CONNECTED)
-		CAMEL_SERVICE_GET_CLASS (service)->disconnect (service, TRUE, NULL);
+		CAMEL_SERVICE_GET_CLASS (service)->disconnect_sync (
+			service, TRUE, NULL, NULL);
 
 	if (service->url)
 		camel_url_free (service->url);
@@ -100,7 +99,7 @@ service_construct (CamelService *service,
 	}
 
 	service->provider = provider;
-	service->url = camel_url_copy(url);
+	service->url = camel_url_copy (url);
 	service->session = g_object_ref (session);
 
 	service->status = CAMEL_SERVICE_DISCONNECTED;
@@ -108,48 +107,16 @@ service_construct (CamelService *service,
 	return TRUE;
 
 fail:
-	url_string = camel_url_to_string(url, CAMEL_URL_HIDE_PASSWORD);
+	url_string = camel_url_to_string (url, CAMEL_URL_HIDE_PASSWORD);
 	g_set_error (
 		error, CAMEL_SERVICE_ERROR,
 		CAMEL_SERVICE_ERROR_URL_INVALID,
 		err, url_string);
-	g_free(url_string);
+	g_free (url_string);
 
 	return FALSE;
 }
 
-static gboolean
-service_connect (CamelService *service,
-                 GError **error)
-{
-	/* Things like the CamelMboxStore can validly
-	 * not define a connect function. */
-	 return TRUE;
-}
-
-static gboolean
-service_disconnect (CamelService *service,
-                    gboolean clean,
-                    GError **error)
-{
-	/* We let people get away with not having a disconnect
-	 * function -- CamelMboxStore, for example. */
-	return TRUE;
-}
-
-static void
-service_cancel_connect (CamelService *service)
-{
-	camel_operation_cancel (service->connect_op);
-}
-
-static GList *
-service_query_auth_types (CamelService *service,
-                          GError **error)
-{
-	return NULL;
-}
-
 static gchar *
 service_get_name (CamelService *service,
                   gboolean brief)
@@ -166,84 +133,77 @@ service_get_path (CamelService *service)
 {
 	CamelProvider *prov = service->provider;
 	CamelURL *url = service->url;
-	GString *use_path1 = NULL, *use_path2 = NULL;
-	gchar *ret_path = NULL;
+	GString *gpath;
+	gchar *path;
 
 	/* A sort of ad-hoc default implementation that works for our
 	 * current set of services.
 	 */
 
+	gpath = g_string_new (service->provider->protocol);
 	if (CAMEL_PROVIDER_ALLOWS (prov, CAMEL_URL_PART_USER)) {
-		use_path1 = g_string_new ("");
-
 		if (CAMEL_PROVIDER_ALLOWS (prov, CAMEL_URL_PART_HOST)) {
-			g_string_append_printf (use_path1, "%s@%s",
+			g_string_append_printf (gpath, "/%s@%s",
 						url->user ? url->user : "",
 						url->host ? url->host : "");
 
 			if (url->port)
-				g_string_append_printf (use_path1, ":%d", url->port);
+				g_string_append_printf (gpath, ":%d", url->port);
 		} else {
-			g_string_append_printf (use_path1, "%s%s", url->user ? url->user : "",
+			g_string_append_printf (gpath, "/%s%s", url->user ? url->user : "",
 						CAMEL_PROVIDER_NEEDS (prov, CAMEL_URL_PART_USER) ? "" : "@");
 		}
-
-		e_filename_make_safe (use_path1->str);
 	} else if (CAMEL_PROVIDER_ALLOWS (prov, CAMEL_URL_PART_HOST)) {
-		use_path1 = g_string_new ("");
-
-		g_string_append_printf (use_path1, "%s%s",
+		g_string_append_printf (gpath, "/%s%s",
 					CAMEL_PROVIDER_NEEDS (prov, CAMEL_URL_PART_HOST) ? "" : "@",
 					url->host ? url->host : "");
 
 		if (url->port)
-			g_string_append_printf (use_path1, ":%d", url->port);
-
-		e_filename_make_safe (use_path1->str);
+			g_string_append_printf (gpath, ":%d", url->port);
 	}
 
-	if (CAMEL_PROVIDER_NEEDS (prov, CAMEL_URL_PART_PATH) && url->path && *url->path) {
-		use_path2 = g_string_new (*url->path == '/' ? url->path + 1 : url->path);
+	if (CAMEL_PROVIDER_NEEDS (prov, CAMEL_URL_PART_PATH))
+		g_string_append_printf (gpath, "%s%s", *url->path == '/' ? "" : "/", url->path);
 
-		/* fix directory separators, if needed */
-		if (G_DIR_SEPARATOR != '/') {
-			gchar **elems = g_strsplit (use_path2->str, "/", -1);
+	path = gpath->str;
+	g_string_free (gpath, FALSE);
 
-			if (elems) {
-				gint ii;
+	return path;
+}
 
-				g_string_truncate (use_path2, 0);
+static void
+service_cancel_connect (CamelService *service)
+{
+	g_cancellable_cancel (service->connect_op);
+}
 
-				for (ii = 0; elems[ii]; ii++) {
-					gchar *elem = elems[ii];
+static gboolean
+service_connect_sync (CamelService *service,
+                      GCancellable *cancellable,
+                      GError **error)
+{
+	/* Things like the CamelMboxStore can validly
+	 * not define a connect function. */
+	 return TRUE;
+}
 
-					if (*elem) {
-						e_filename_make_safe (elem);
+static gboolean
+service_disconnect_sync (CamelService *service,
+                         gboolean clean,
+                         GCancellable *cancellable,
+                         GError **error)
+{
+	/* We let people get away with not having a disconnect
+	 * function -- CamelMboxStore, for example. */
+	return TRUE;
+}
 
-						if (use_path2->len)
-							g_string_append_c (use_path2, G_DIR_SEPARATOR);
-						g_string_append (use_path2, elem);
-					}
-				}
-
-				g_strfreev (elems);
-			}
-		}
-	}
-
-	if (!use_path1 && use_path2) {
-		use_path1 = use_path2;
-		use_path2 = NULL;
-	}
-
-	ret_path = g_build_filename (service->provider->protocol, use_path1 ? use_path1->str : NULL, use_path2 ? use_path2->str : NULL, NULL);
-
-	if (use_path1)
-		g_string_free (use_path1, TRUE);
-	if (use_path2)
-		g_string_free (use_path2, TRUE);
-
-	return ret_path;
+static GList *
+service_query_auth_types_sync (CamelService *service,
+                               GCancellable *cancellable,
+                               GError **error)
+{
+	return NULL;
 }
 
 static void
@@ -257,12 +217,12 @@ camel_service_class_init (CamelServiceClass *class)
 	object_class->finalize = service_finalize;
 
 	class->construct = service_construct;
-	class->connect = service_connect;
-	class->disconnect = service_disconnect;
-	class->cancel_connect = service_cancel_connect;
-	class->query_auth_types = service_query_auth_types;
 	class->get_name = service_get_name;
 	class->get_path = service_get_path;
+	class->cancel_connect = service_cancel_connect;
+	class->connect_sync = service_connect_sync;
+	class->disconnect_sync = service_disconnect_sync;
+	class->query_auth_types_sync = service_query_auth_types_sync;
 }
 
 static void
@@ -321,130 +281,6 @@ camel_service_construct (CamelService *service,
 	return success;
 }
 
-/**
- * camel_service_connect:
- * @service: a #CamelService object
- * @error: return location for a #GError, or %NULL
- *
- * Connect to the service using the parameters it was initialized
- * with.
- *
- * Returns: %TRUE if the connection is made or %FALSE otherwise
- **/
-gboolean
-camel_service_connect (CamelService *service,
-                       GError **error)
-{
-	CamelServiceClass *class;
-	gboolean ret = FALSE;
-	gboolean unreg = FALSE;
-	CamelOperation *connect_op;
-
-	g_return_val_if_fail (CAMEL_IS_SERVICE (service), FALSE);
-	g_return_val_if_fail (service->session != NULL, FALSE);
-	g_return_val_if_fail (service->url != NULL, FALSE);
-
-	class = CAMEL_SERVICE_GET_CLASS (service);
-	g_return_val_if_fail (class->connect != NULL, FALSE);
-
-	camel_service_lock (service, CAMEL_SERVICE_REC_CONNECT_LOCK);
-
-	if (service->status == CAMEL_SERVICE_CONNECTED) {
-		camel_service_unlock (service, CAMEL_SERVICE_REC_CONNECT_LOCK);
-		return TRUE;
-	}
-
-	/* Register a separate operation for connecting, so that
-	 * the offline code can cancel it. */
-	camel_service_lock (service, CAMEL_SERVICE_CONNECT_OP_LOCK);
-	service->connect_op = camel_operation_registered ();
-	if (!service->connect_op) {
-		service->connect_op = camel_operation_new (NULL, NULL);
-		camel_operation_register (service->connect_op);
-		unreg = TRUE;
-	}
-	connect_op = service->connect_op;
-	camel_service_unlock (service, CAMEL_SERVICE_CONNECT_OP_LOCK);
-
-	service->status = CAMEL_SERVICE_CONNECTING;
-	ret = class->connect (service, error);
-	CAMEL_CHECK_GERROR (service, connect, ret, error);
-	service->status = ret ? CAMEL_SERVICE_CONNECTED : CAMEL_SERVICE_DISCONNECTED;
-
-	camel_service_lock (service, CAMEL_SERVICE_CONNECT_OP_LOCK);
-	if (connect_op) {
-		if (unreg && service->connect_op)
-			camel_operation_unregister (connect_op);
-
-		camel_operation_unref (connect_op);
-		service->connect_op = NULL;
-	}
-	camel_service_unlock (service, CAMEL_SERVICE_CONNECT_OP_LOCK);
-
-	camel_service_unlock (service, CAMEL_SERVICE_REC_CONNECT_LOCK);
-
-	return ret;
-}
-
-/**
- * camel_service_disconnect:
- * @service: a #CamelService object
- * @clean: whether or not to try to disconnect cleanly
- * @error: return location for a #GError, or %NULL
- *
- * Disconnect from the service. If @clean is %FALSE, it should not
- * try to do any synchronizing or other cleanup of the connection.
- *
- * Returns: %TRUE if the disconnect was successful or %FALSE otherwise
- **/
-gboolean
-camel_service_disconnect (CamelService *service,
-                          gboolean clean,
-                          GError **error)
-{
-	CamelServiceClass *class;
-	gboolean res = TRUE;
-	gint unreg = FALSE;
-
-	g_return_val_if_fail (CAMEL_IS_SERVICE (service), FALSE);
-
-	class = CAMEL_SERVICE_GET_CLASS (service);
-	g_return_val_if_fail (class->disconnect != NULL, FALSE);
-
-	camel_service_lock (service, CAMEL_SERVICE_REC_CONNECT_LOCK);
-
-	if (service->status != CAMEL_SERVICE_DISCONNECTED
-	    && service->status != CAMEL_SERVICE_DISCONNECTING) {
-		camel_service_lock (service, CAMEL_SERVICE_CONNECT_OP_LOCK);
-		service->connect_op = camel_operation_registered ();
-		if (!service->connect_op) {
-			service->connect_op = camel_operation_new (NULL, NULL);
-			camel_operation_register (service->connect_op);
-			unreg = TRUE;
-		}
-		camel_service_unlock (service, CAMEL_SERVICE_CONNECT_OP_LOCK);
-
-		service->status = CAMEL_SERVICE_DISCONNECTING;
-		res = class->disconnect (service, clean, error);
-		CAMEL_CHECK_GERROR (service, disconnect, res, error);
-		service->status = CAMEL_SERVICE_DISCONNECTED;
-
-		camel_service_lock (service, CAMEL_SERVICE_CONNECT_OP_LOCK);
-		if (unreg)
-			camel_operation_unregister (service->connect_op);
-
-		camel_operation_unref (service->connect_op);
-		service->connect_op = NULL;
-		camel_service_unlock (service, CAMEL_SERVICE_CONNECT_OP_LOCK);
-	}
-
-	camel_service_unlock (service, CAMEL_SERVICE_REC_CONNECT_LOCK);
-
-	service->status = CAMEL_SERVICE_DISCONNECTED;
-
-	return res;
-}
-
 /**
  * camel_service_cancel_connect:
  * @service: a #CamelService object
@@ -469,24 +305,6 @@ camel_service_cancel_connect (CamelService *service)
 	camel_service_unlock (service, CAMEL_SERVICE_CONNECT_OP_LOCK);
 }
 
-/**
- * camel_service_get_url:
- * @service: a #CamelService object
- *
- * Gets the URL representing @service. The returned URL must be
- * freed when it is no longer needed. For security reasons, this
- * routine does not return the password.
- *
- * Returns: the URL representing @service
- **/
-gchar *
-camel_service_get_url (CamelService *service)
-{
-	g_return_val_if_fail (CAMEL_IS_SERVICE (service), NULL);
-
-	return camel_url_to_string (service->url, CAMEL_URL_HIDE_PASSWORD);
-}
-
 /**
  * camel_service_get_name:
  * @service: a #CamelService object
@@ -573,8 +391,130 @@ camel_service_get_provider (CamelService *service)
 }
 
 /**
- * camel_service_query_auth_types:
+ * camel_service_get_url:
  * @service: a #CamelService object
+ *
+ * Gets the URL representing @service. The returned URL must be
+ * freed when it is no longer needed. For security reasons, this
+ * routine does not return the password.
+ *
+ * Returns: the URL representing @service
+ **/
+gchar *
+camel_service_get_url (CamelService *service)
+{
+	g_return_val_if_fail (CAMEL_IS_SERVICE (service), NULL);
+
+	return camel_url_to_string (service->url, CAMEL_URL_HIDE_PASSWORD);
+}
+
+/**
+ * camel_service_connect_sync:
+ * @service: a #CamelService object
+ * @error: return location for a #GError, or %NULL
+ *
+ * Connect to the service using the parameters it was initialized
+ * with.
+ *
+ * Returns: %TRUE if the connection is made or %FALSE otherwise
+ **/
+gboolean
+camel_service_connect_sync (CamelService *service,
+                            GError **error)
+{
+	CamelServiceClass *class;
+	gboolean ret = FALSE;
+
+	g_return_val_if_fail (CAMEL_IS_SERVICE (service), FALSE);
+	g_return_val_if_fail (service->session != NULL, FALSE);
+	g_return_val_if_fail (service->url != NULL, FALSE);
+
+	class = CAMEL_SERVICE_GET_CLASS (service);
+	g_return_val_if_fail (class->connect_sync != NULL, FALSE);
+
+	camel_service_lock (service, CAMEL_SERVICE_REC_CONNECT_LOCK);
+
+	if (service->status == CAMEL_SERVICE_CONNECTED) {
+		camel_service_unlock (service, CAMEL_SERVICE_REC_CONNECT_LOCK);
+		return TRUE;
+	}
+
+	/* Register a separate operation for connecting, so that
+	 * the offline code can cancel it. */
+	camel_service_lock (service, CAMEL_SERVICE_CONNECT_OP_LOCK);
+	service->connect_op = camel_operation_new ();
+	camel_service_unlock (service, CAMEL_SERVICE_CONNECT_OP_LOCK);
+
+	service->status = CAMEL_SERVICE_CONNECTING;
+	ret = class->connect_sync (service, service->connect_op, error);
+	CAMEL_CHECK_GERROR (service, connect_sync, ret, error);
+	service->status = ret ? CAMEL_SERVICE_CONNECTED : CAMEL_SERVICE_DISCONNECTED;
+
+	camel_service_lock (service, CAMEL_SERVICE_CONNECT_OP_LOCK);
+	g_object_unref (service->connect_op);
+	service->connect_op = NULL;
+	camel_service_unlock (service, CAMEL_SERVICE_CONNECT_OP_LOCK);
+
+	camel_service_unlock (service, CAMEL_SERVICE_REC_CONNECT_LOCK);
+
+	return ret;
+}
+
+/**
+ * camel_service_disconnect_sync:
+ * @service: a #CamelService object
+ * @clean: whether or not to try to disconnect cleanly
+ * @error: return location for a #GError, or %NULL
+ *
+ * Disconnect from the service. If @clean is %FALSE, it should not
+ * try to do any synchronizing or other cleanup of the connection.
+ *
+ * Returns: %TRUE if the disconnect was successful or %FALSE otherwise
+ **/
+gboolean
+camel_service_disconnect_sync (CamelService *service,
+                               gboolean clean,
+                               GError **error)
+{
+	CamelServiceClass *class;
+	gboolean res = TRUE;
+
+	g_return_val_if_fail (CAMEL_IS_SERVICE (service), FALSE);
+
+	class = CAMEL_SERVICE_GET_CLASS (service);
+	g_return_val_if_fail (class->disconnect_sync != NULL, FALSE);
+
+	camel_service_lock (service, CAMEL_SERVICE_REC_CONNECT_LOCK);
+
+	if (service->status != CAMEL_SERVICE_DISCONNECTED
+	    && service->status != CAMEL_SERVICE_DISCONNECTING) {
+		camel_service_lock (service, CAMEL_SERVICE_CONNECT_OP_LOCK);
+		service->connect_op = camel_operation_new ();
+		camel_service_unlock (service, CAMEL_SERVICE_CONNECT_OP_LOCK);
+
+		service->status = CAMEL_SERVICE_DISCONNECTING;
+		res = class->disconnect_sync (
+			service, clean, service->connect_op, error);
+		CAMEL_CHECK_GERROR (service, disconnect_sync, res, error);
+		service->status = CAMEL_SERVICE_DISCONNECTED;
+
+		camel_service_lock (service, CAMEL_SERVICE_CONNECT_OP_LOCK);
+		g_object_unref (service->connect_op);
+		service->connect_op = NULL;
+		camel_service_unlock (service, CAMEL_SERVICE_CONNECT_OP_LOCK);
+	}
+
+	camel_service_unlock (service, CAMEL_SERVICE_REC_CONNECT_LOCK);
+
+	service->status = CAMEL_SERVICE_DISCONNECTED;
+
+	return res;
+}
+
+/**
+ * camel_service_query_auth_types_sync:
+ * @service: a #CamelService object
+ * @cancellable: optional #GCancellable object, or %NULL
  * @error: return location for a #GError, or %NULL
  *
  * This is used by the mail source wizard to get the list of
@@ -585,8 +525,9 @@ camel_service_get_provider (CamelService *service)
  * must free the list with #g_list_free when it is done with it.
  **/
 GList *
-camel_service_query_auth_types (CamelService *service,
-                                GError **error)
+camel_service_query_auth_types_sync (CamelService *service,
+                                     GCancellable *cancellable,
+                                     GError **error)
 {
 	CamelServiceClass *class;
 	GList *ret;
@@ -594,12 +535,12 @@ camel_service_query_auth_types (CamelService *service,
 	g_return_val_if_fail (CAMEL_IS_SERVICE (service), NULL);
 
 	class = CAMEL_SERVICE_GET_CLASS (service);
-	g_return_val_if_fail (class->query_auth_types != NULL, NULL);
+	g_return_val_if_fail (class->query_auth_types_sync != NULL, NULL);
 
 	/* Note that we get the connect lock here, which means the
 	 * callee must not call the connect functions itself. */
 	camel_service_lock (service, CAMEL_SERVICE_REC_CONNECT_LOCK);
-	ret = class->query_auth_types (service, error);
+	ret = class->query_auth_types_sync (service, cancellable, error);
 	camel_service_unlock (service, CAMEL_SERVICE_REC_CONNECT_LOCK);
 
 	return ret;
