@@ -39,6 +39,11 @@
 
 #include <glib/gi18n-lib.h>
 
+#ifdef G_OS_WIN32
+#include <winsock2.h>
+#include <ws2tcpip.h>
+#endif
+
 #include "camel-file-utils.h"
 #include "camel-net-utils.h"
 #include "camel-operation.h"
@@ -237,8 +242,13 @@ _set_errno_from_pr_error (gint pr_code)
 void
 _set_g_error_from_errno (GError **error, gboolean eintr_means_cancelled)
 {
+	gint errn = errno;
+
+	if (error)
+		g_clear_error (error);
+
 	/* This is stolen from camel_read() / camel_write() */
-	if (eintr_means_cancelled && errno == EINTR)
+	if (eintr_means_cancelled && errn == EINTR)
 		g_set_error (
 			error, G_IO_ERROR,
 			G_IO_ERROR_CANCELLED,
@@ -246,23 +256,30 @@ _set_g_error_from_errno (GError **error, gboolean eintr_means_cancelled)
 	else
 		g_set_error (
 			error, G_IO_ERROR,
-			g_io_error_from_errno (errno),
-			"%s", g_strerror (errno));
+			g_io_error_from_errno (errn),
+			"%s", g_strerror (errn));
 }
 
 static gssize
-read_from_prfd (PRFileDesc *fd, gchar *buffer, gsize n, GError **error)
+read_from_prfd (PRFileDesc *fd,
+                gchar *buffer,
+                gsize n,
+                GCancellable *cancellable,
+                GError **error)
 {
-	PRFileDesc *cancel_fd;
+	PRFileDesc *cancel_fd = NULL;
 	gssize nread;
 
-	if (camel_operation_cancel_check (NULL)) {
+	if (g_cancellable_is_cancelled (cancellable)) {
 		errno = EINTR;
 		_set_g_error_from_errno (error, TRUE);
 		return -1;
 	}
 
-	cancel_fd = camel_operation_cancel_prfd (NULL);
+	if (CAMEL_IS_OPERATION (cancellable))
+		cancel_fd = camel_operation_cancel_prfd (
+			CAMEL_OPERATION (cancellable));
+
 	if (cancel_fd == NULL) {
 		do {
 			nread = PR_Read (fd, buffer, n);
@@ -297,9 +314,9 @@ read_from_prfd (PRFileDesc *fd, gchar *buffer, gsize n, GError **error)
 			pollfds[1].out_flags = 0;
 			nread = -1;
 
-			res = PR_Poll(pollfds, 2, IO_TIMEOUT);
+			res = PR_Poll (pollfds, 2, IO_TIMEOUT);
 			if (res == -1)
-				_set_errno_from_pr_error (PR_GetError());
+				_set_errno_from_pr_error (PR_GetError ());
 			else if (res == 0) {
 #ifdef ETIMEDOUT
 				errno = ETIMEDOUT;
@@ -340,27 +357,35 @@ static gssize
 tcp_stream_raw_read (CamelStream *stream,
                      gchar *buffer,
                      gsize n,
+                     GCancellable *cancellable,
                      GError **error)
 {
 	CamelTcpStreamRaw *raw = CAMEL_TCP_STREAM_RAW (stream);
 	CamelTcpStreamRawPrivate *priv = raw->priv;
 
-	return read_from_prfd (priv->sockfd, buffer, n, error);
+	return read_from_prfd (priv->sockfd, buffer, n, cancellable, error);
 }
 
 static gssize
-write_to_prfd (PRFileDesc *fd, const gchar *buffer, gsize n, GError **error)
+write_to_prfd (PRFileDesc *fd,
+               const gchar *buffer,
+               gsize n,
+               GCancellable *cancellable,
+               GError **error)
 {
+	PRFileDesc *cancel_fd = NULL;
 	gssize w, written = 0;
-	PRFileDesc *cancel_fd;
 
-	if (camel_operation_cancel_check (NULL)) {
+	if (g_cancellable_is_cancelled (cancellable)) {
 		errno = EINTR;
 		_set_g_error_from_errno (error, TRUE);
 		return -1;
 	}
 
-	cancel_fd = camel_operation_cancel_prfd (NULL);
+	if (CAMEL_IS_OPERATION (cancellable))
+		cancel_fd = camel_operation_cancel_prfd (
+			CAMEL_OPERATION (cancellable));
+
 	if (cancel_fd == NULL) {
 		do {
 			do {
@@ -402,7 +427,7 @@ write_to_prfd (PRFileDesc *fd, const gchar *buffer, gsize n, GError **error)
 
 			res = PR_Poll (pollfds, 2, IO_TIMEOUT);
 			if (res == -1) {
-				_set_errno_from_pr_error (PR_GetError());
+				_set_errno_from_pr_error (PR_GetError ());
 				if (PR_GetError () == PR_PENDING_INTERRUPT_ERROR)
 					w = 0;
 			} else if (res == 0) {
@@ -447,16 +472,18 @@ static gssize
 tcp_stream_raw_write (CamelStream *stream,
                       const gchar *buffer,
                       gsize n,
+                      GCancellable *cancellable,
                       GError **error)
 {
 	CamelTcpStreamRaw *raw = CAMEL_TCP_STREAM_RAW (stream);
 	CamelTcpStreamRawPrivate *priv = raw->priv;
 
-	return write_to_prfd (priv->sockfd, buffer, n, error);
+	return write_to_prfd (priv->sockfd, buffer, n, cancellable, error);
 }
 
 static gint
 tcp_stream_raw_flush (CamelStream *stream,
+                      GCancellable *cancellable,
                       GError **error)
 {
 #if 0
@@ -470,6 +497,7 @@ tcp_stream_raw_flush (CamelStream *stream,
 
 static gint
 tcp_stream_raw_close (CamelStream *stream,
+                      GCancellable *cancellable,
                       GError **error)
 {
 	CamelTcpStreamRaw *raw = CAMEL_TCP_STREAM_RAW (stream);
@@ -486,7 +514,7 @@ tcp_stream_raw_close (CamelStream *stream,
 		priv->sockfd = NULL;
 
 		if (err)
-			_set_errno_from_pr_error (PR_GetError());
+			_set_errno_from_pr_error (PR_GetError ());
 		else
 			return 0;
 	}
@@ -501,17 +529,17 @@ sockaddr_to_praddr (struct sockaddr *s, gint len, PRNetAddr *addr)
 	/* We assume the ip addresses are the same size - they have to be anyway.
 	   We could probably just use memcpy *shrug* */
 
-	memset(addr, 0, sizeof(*addr));
+	memset (addr, 0, sizeof (*addr));
 
 	if (s->sa_family == AF_INET) {
 		struct sockaddr_in *sin = (struct sockaddr_in *)s;
 
-		if (len < sizeof(*sin))
+		if (len < sizeof (*sin))
 			return -1;
 
 		addr->inet.family = PR_AF_INET;
 		addr->inet.port = sin->sin_port;
-		memcpy(&addr->inet.ip, &sin->sin_addr, sizeof(addr->inet.ip));
+		memcpy (&addr->inet.ip, &sin->sin_addr, sizeof (addr->inet.ip));
 
 		return 0;
 	}
@@ -519,13 +547,13 @@ sockaddr_to_praddr (struct sockaddr *s, gint len, PRNetAddr *addr)
 	else if (s->sa_family == PR_AF_INET6) {
 		struct sockaddr_in6 *sin = (struct sockaddr_in6 *)s;
 
-		if (len < sizeof(*sin))
+		if (len < sizeof (*sin))
 			return -1;
 
 		addr->ipv6.family = PR_AF_INET6;
 		addr->ipv6.port = sin->sin6_port;
 		addr->ipv6.flowinfo = sin->sin6_flowinfo;
-		memcpy(&addr->ipv6.ip, &sin->sin6_addr, sizeof(addr->ipv6.ip));
+		memcpy (&addr->ipv6.ip, &sin->sin6_addr, sizeof (addr->ipv6.ip));
 		addr->ipv6.scope_id = sin->sin6_scope_id;
 
 		return 0;
@@ -536,25 +564,29 @@ sockaddr_to_praddr (struct sockaddr *s, gint len, PRNetAddr *addr)
 }
 
 static PRFileDesc *
-socket_connect (struct addrinfo *host, GError **error)
+socket_connect (struct addrinfo *host,
+                GCancellable *cancellable,
+                GError **error)
 {
 	PRNetAddr netaddr;
-	PRFileDesc *fd, *cancel_fd;
+	PRFileDesc *fd, *cancel_fd = NULL;
 
-	if (sockaddr_to_praddr(host->ai_addr, host->ai_addrlen, &netaddr) != 0) {
+	if (sockaddr_to_praddr (host->ai_addr, host->ai_addrlen, &netaddr) != 0) {
 		errno = EINVAL;
 		_set_g_error_from_errno (error, FALSE);
 		return NULL;
 	}
 
-	fd = PR_OpenTCPSocket(netaddr.raw.family);
+	fd = PR_OpenTCPSocket (netaddr.raw.family);
 	if (fd == NULL) {
 		_set_errno_from_pr_error (PR_GetError ());
 		_set_g_error_from_errno (error, FALSE);
 		return NULL;
 	}
 
-	cancel_fd = camel_operation_cancel_prfd(NULL);
+	if (CAMEL_IS_OPERATION (cancellable))
+		cancel_fd = camel_operation_cancel_prfd (
+			CAMEL_OPERATION (cancellable));
 
 	if (PR_Connect (fd, &netaddr, cancel_fd?0:CONNECT_TIMEOUT) == PR_FAILURE) {
 		gint errnosave;
@@ -585,7 +617,7 @@ socket_connect (struct addrinfo *host, GError **error)
 					goto exception;
 				}
 
-				if (PR_ConnectContinue(fd, poll[0].out_flags) == PR_FAILURE) {
+				if (PR_ConnectContinue (fd, poll[0].out_flags) == PR_FAILURE) {
 					_set_errno_from_pr_error (PR_GetError ());
 					if (PR_GetError () != PR_IN_PROGRESS_ERROR)
 						goto exception;
@@ -619,7 +651,11 @@ out:
  * negotiate anything with the proxy; this is just to create the socket and connect.
  */
 static PRFileDesc *
-connect_to_proxy (CamelTcpStreamRaw *raw, const gchar *proxy_host, gint proxy_port, GError **error)
+connect_to_proxy (CamelTcpStreamRaw *raw,
+                  const gchar *proxy_host,
+                  gint proxy_port,
+                  GCancellable *cancellable,
+                  GError **error)
 {
 	struct addrinfo *addr, *ai, hints;
 	gchar serv[16];
@@ -635,7 +671,8 @@ connect_to_proxy (CamelTcpStreamRaw *raw, const gchar *proxy_host, gint proxy_po
 	memset (&hints, 0, sizeof (hints));
 	hints.ai_socktype = SOCK_STREAM;
 
-	addr = camel_getaddrinfo (proxy_host, serv, &hints, error);
+	addr = camel_getaddrinfo (
+		proxy_host, serv, &hints, cancellable, error);
 	if (!addr)
 		return NULL;
 
@@ -643,7 +680,7 @@ connect_to_proxy (CamelTcpStreamRaw *raw, const gchar *proxy_host, gint proxy_po
 
 	ai = addr;
 	while (ai) {
-		fd = socket_connect (ai, error);
+		fd = socket_connect (ai, cancellable, error);
 		if (fd)
 			goto out;
 
@@ -669,17 +706,23 @@ out:
  * connect_addr; if you want to traverse all the addrinfos, call this function for each of them.
  */
 static PRFileDesc *
-connect_to_socks4_proxy (CamelTcpStreamRaw *raw, const gchar *proxy_host, gint proxy_port, struct addrinfo *connect_addr, GError **error)
+connect_to_socks4_proxy (CamelTcpStreamRaw *raw,
+                         const gchar *proxy_host,
+                         gint proxy_port,
+                         struct addrinfo *connect_addr,
+                         GCancellable *cancellable,
+                         GError **error)
 {
 	PRFileDesc *fd;
 	gchar request[9];
 	struct sockaddr_in *sin;
-	gchar reply[8];
+	gchar reply[8]; /* note that replies are 8 bytes, even if only the first 2 are used */
 	gint save_errno;
 
-	g_assert (connect_addr->ai_addr->sa_family == AF_INET); /* FMQ: check for AF_INET in the caller */
+	g_assert (connect_addr->ai_addr->sa_family == AF_INET);
 
-	fd = connect_to_proxy (raw, proxy_host, proxy_port, error);
+	fd = connect_to_proxy (
+		raw, proxy_host, proxy_port, cancellable, error);
 	if (!fd)
 		goto error;
 
@@ -692,25 +735,39 @@ connect_to_socks4_proxy (CamelTcpStreamRaw *raw, const gchar *proxy_host, gint p
 	request[8] = 0x00;				/* terminator */
 
 	d (g_print ("  writing SOCKS4 request to connect to actual host\n"));
-	if (write_to_prfd (fd, request, sizeof (request), error) != sizeof (request)) {
+	if (write_to_prfd (fd, request, sizeof (request), cancellable, error) != sizeof (request)) {
 		d (g_print ("  failed: %d\n", errno));
 		goto error;
 	}
 
 	d (g_print ("  reading SOCKS4 reply\n"));
-	if (read_from_prfd (fd, reply, sizeof (reply), error) != sizeof (reply)) {
+	if (read_from_prfd (fd, reply, sizeof (reply), cancellable, error) != sizeof (reply)) {
 		d (g_print ("  failed: %d\n", errno));
+		g_set_error (error, CAMEL_PROXY_ERROR, CAMEL_PROXY_ERROR_PROXY_NOT_SUPPORTED,
+			     _("The proxy host does not support SOCKS4"));
 		goto error;
 	}
 
-	if (!(reply[0] == 0		/* first byte of reply is 0 */
-	      && reply[1] == 90)) {	/* 90 means "request granted" */
+	if (reply[0] != 0) { /* version of reply code is 0 */
 #ifdef G_OS_WIN32
 		errno = WSAECONNREFUSED;
 #else
 		errno = ECONNREFUSED;
 #endif
-		_set_g_error_from_errno (error, FALSE);
+		g_set_error (error, CAMEL_PROXY_ERROR, CAMEL_PROXY_ERROR_PROXY_NOT_SUPPORTED,
+			     _("The proxy host does not support SOCKS4"));
+		goto error;
+	}
+
+	if (reply[1] != 90) {	/* 90 means "request granted" */
+#ifdef G_OS_WIN32
+		errno = WSAECONNREFUSED;
+#else
+		errno = ECONNREFUSED;
+#endif
+		g_set_error (error, CAMEL_PROXY_ERROR, CAMEL_PROXY_ERROR_CANT_AUTHENTICATE,
+			     _("The proxy host denied our request: code %d"),
+			     reply[1]);
 		goto error;
 	}
 
@@ -738,7 +795,10 @@ out:
 
 /* Resolves a port number using getaddrinfo().  Returns 0 if the port can't be resolved or if the operation is cancelled */
 static gint
-resolve_port (const char *service, gint fallback_port, GError **error)
+resolve_port (const gchar *service,
+              gint fallback_port,
+              GCancellable *cancellable,
+              GError **error)
 {
 	struct addrinfo *ai;
 	GError *my_error;
@@ -751,7 +811,8 @@ resolve_port (const char *service, gint fallback_port, GError **error)
 	 * from the standard getaddrinfo(), which lets you pass a NULL hostname
 	 * if you just want to resolve a port number.
 	 */
-	ai = camel_getaddrinfo ("localhost", service, NULL, &my_error);
+	ai = camel_getaddrinfo (
+		"localhost", service, NULL, cancellable, &my_error);
 	if (ai == NULL && fallback_port != 0 && !g_error_matches (my_error, G_IO_ERROR, G_IO_ERROR_CANCELLED))
 		port = fallback_port;
 	else if (ai == NULL) {
@@ -778,7 +839,10 @@ resolve_port (const char *service, gint fallback_port, GError **error)
 }
 
 static gboolean
-socks5_initiate_and_request_authentication (CamelTcpStreamRaw *raw, PRFileDesc *fd, GError **error)
+socks5_initiate_and_request_authentication (CamelTcpStreamRaw *raw,
+                                            PRFileDesc *fd,
+                                            GCancellable *cancellable,
+                                            GError **error)
 {
 	gchar request[3];
 	gchar reply[2];
@@ -788,14 +852,17 @@ socks5_initiate_and_request_authentication (CamelTcpStreamRaw *raw, PRFileDesc *
 	request[2] = 0;		/* no authentication, please - extending this is left as an exercise for the reader */
 
 	d (g_print ("  writing SOCKS5 request for authentication\n"));
-	if (write_to_prfd (fd, request, sizeof (request), error) != sizeof (request)) {
+	if (write_to_prfd (fd, request, sizeof (request), cancellable, error) != sizeof (request)) {
 		d (g_print ("  failed: %d\n", errno));
 		return FALSE;
 	}
 
 	d (g_print ("  reading SOCKS5 reply\n"));
-	if (read_from_prfd (fd, reply, sizeof (reply), error) != sizeof (reply)) {
+	if (read_from_prfd (fd, reply, sizeof (reply), cancellable, error) != sizeof (reply)) {
 		d (g_print ("  failed: %d\n", errno));
+		g_clear_error (error);
+		g_set_error (error, CAMEL_PROXY_ERROR, CAMEL_PROXY_ERROR_PROXY_NOT_SUPPORTED,
+			     _("The proxy host does not support SOCKS5"));
 		return FALSE;
 	}
 
@@ -815,7 +882,7 @@ socks5_initiate_and_request_authentication (CamelTcpStreamRaw *raw, PRFileDesc *
 	return TRUE;
 }
 
-static const char *
+static const gchar *
 socks5_reply_error_to_string (gchar error_code)
 {
 	switch (error_code) {
@@ -832,7 +899,10 @@ socks5_reply_error_to_string (gchar error_code)
 }
 
 static gboolean
-socks5_consume_reply_address (CamelTcpStreamRaw *raw, PRFileDesc *fd, GError **error)
+socks5_consume_reply_address (CamelTcpStreamRaw *raw,
+                              PRFileDesc *fd,
+                              GCancellable *cancellable,
+                              GError **error)
 {
 	gchar address_type;
 	gint bytes_to_consume;
@@ -840,7 +910,7 @@ socks5_consume_reply_address (CamelTcpStreamRaw *raw, PRFileDesc *fd, GError **e
 
 	address_and_port = NULL;
 
-	if (read_from_prfd (fd, &address_type, sizeof (address_type), error) != sizeof (address_type))
+	if (read_from_prfd (fd, &address_type, sizeof (address_type), cancellable, error) != sizeof (address_type))
 		goto incomplete_reply;
 
 	if (address_type == 0x01)
@@ -852,7 +922,7 @@ socks5_consume_reply_address (CamelTcpStreamRaw *raw, PRFileDesc *fd, GError **e
 
 		/* we'll get an octet with the address length, and then the address itself */
 
-		if (read_from_prfd (fd, (gchar *) &address_len, sizeof (address_len), error) != sizeof (address_len))
+		if (read_from_prfd (fd, (gchar *) &address_len, sizeof (address_len), cancellable, error) != sizeof (address_len))
 			goto incomplete_reply;
 
 		bytes_to_consume = address_len;
@@ -864,7 +934,7 @@ socks5_consume_reply_address (CamelTcpStreamRaw *raw, PRFileDesc *fd, GError **e
 	bytes_to_consume += 2; /* 2 octets for port number */
 	address_and_port = g_new (gchar, bytes_to_consume);
 
-	if (read_from_prfd (fd, address_and_port, bytes_to_consume, error) != bytes_to_consume)
+	if (read_from_prfd (fd, address_and_port, bytes_to_consume, cancellable, error) != bytes_to_consume)
 		goto incomplete_reply;
 
 	g_free (address_and_port); /* Currently we don't do anything to these; maybe some authenticated method will need them later */
@@ -874,12 +944,18 @@ socks5_consume_reply_address (CamelTcpStreamRaw *raw, PRFileDesc *fd, GError **e
 incomplete_reply:
 	g_free (address_and_port);
 
+	g_clear_error (error);
 	g_set_error (error, CAMEL_SERVICE_ERROR, CAMEL_SERVICE_ERROR_NOT_CONNECTED, _("Incomplete reply from SOCKS server"));
 	return FALSE;
 }
 
 static gboolean
-socks5_request_connect (CamelTcpStreamRaw *raw, PRFileDesc *fd, const char *host, gint port, GError **error)
+socks5_request_connect (CamelTcpStreamRaw *raw,
+                        PRFileDesc *fd,
+                        const gchar *host,
+                        gint port,
+                        GCancellable *cancellable,
+                        GError **error)
 {
 	gchar *request;
 	gchar reply[3];
@@ -906,7 +982,7 @@ socks5_request_connect (CamelTcpStreamRaw *raw, PRFileDesc *fd, const char *host
 	request[5 + host_len + 1] = port & 0xff;      /* low byte of port */
 
 	d (g_print ("  writing SOCKS5 request for connection\n"));
-	num_written = write_to_prfd (fd, request, request_len, error);
+	num_written = write_to_prfd (fd, request, request_len, cancellable, error);
 	g_free (request);
 
 	if (num_written != request_len) {
@@ -915,7 +991,7 @@ socks5_request_connect (CamelTcpStreamRaw *raw, PRFileDesc *fd, const char *host
 	}
 
 	d (g_print ("  reading SOCKS5 reply\n"));
-	if (read_from_prfd (fd, reply, sizeof (reply), error) != sizeof (reply)) {
+	if (read_from_prfd (fd, reply, sizeof (reply), cancellable, error) != sizeof (reply)) {
 		d (g_print ("  failed: %d\n", errno));
 		return FALSE;
 	}
@@ -939,34 +1015,39 @@ socks5_request_connect (CamelTcpStreamRaw *raw, PRFileDesc *fd, const char *host
 	 * identify to the final host.  This is of variable length, so we must
 	 * consume it by hand.
 	 */
-	if (!socks5_consume_reply_address (raw, fd, error))
+	if (!socks5_consume_reply_address (raw, fd, cancellable, error))
 		return FALSE;
-	
+
 	return TRUE;
 }
 
 /* RFC 1928 - SOCKS protocol version 5 */
 static PRFileDesc *
 connect_to_socks5_proxy (CamelTcpStreamRaw *raw,
-			 const char *proxy_host, gint proxy_port,
-			 const char *host, const char *service, gint fallback_port,
-			 GError **error)
+                         const gchar *proxy_host,
+                         gint proxy_port,
+                         const gchar *host,
+                         const gchar *service,
+                         gint fallback_port,
+                         GCancellable *cancellable,
+                         GError **error)
 {
 	PRFileDesc *fd;
 	gint port;
 
-	fd = connect_to_proxy (raw, proxy_host, proxy_port, error);
+	fd = connect_to_proxy (
+		raw, proxy_host, proxy_port, cancellable, error);
 	if (!fd)
 		goto error;
 
-	port = resolve_port (service, fallback_port, error);
+	port = resolve_port (service, fallback_port, cancellable, error);
 	if (port == 0)
 		goto error;
 
-	if (!socks5_initiate_and_request_authentication (raw, fd, error))
+	if (!socks5_initiate_and_request_authentication (raw, fd, cancellable, error))
 		goto error;
 
-	if (!socks5_request_connect (raw, fd, host, port, error))
+	if (!socks5_request_connect (raw, fd, host, port, cancellable, error))
 		goto error;
 
 	d (g_print ("  success\n"));
@@ -991,12 +1072,15 @@ out:
 	d (g_print ("}\n"));
 
 	return fd;
-	
+
 }
 
 static gint
 tcp_stream_raw_connect (CamelTcpStream *stream,
-			const char *host, const char *service, gint fallback_port,
+                        const gchar *host,
+                        const gchar *service,
+                        gint fallback_port,
+                        GCancellable *cancellable,
                         GError **error)
 {
 	CamelTcpStreamRaw *raw = CAMEL_TCP_STREAM_RAW (stream);
@@ -1014,7 +1098,9 @@ tcp_stream_raw_connect (CamelTcpStream *stream,
 		/* First, try SOCKS5, which does name resolution itself */
 
 		my_error = NULL;
-		priv->sockfd = connect_to_socks5_proxy (raw, proxy_host, proxy_port, host, service, fallback_port, &my_error);
+		priv->sockfd = connect_to_socks5_proxy (
+			raw, proxy_host, proxy_port, host, service,
+			fallback_port, cancellable, &my_error);
 		if (priv->sockfd)
 			return 0;
 		else if (g_error_matches (my_error, CAMEL_PROXY_ERROR, CAMEL_PROXY_ERROR_CANT_AUTHENTICATE)
@@ -1031,13 +1117,15 @@ tcp_stream_raw_connect (CamelTcpStream *stream,
 	hints.ai_family = PF_UNSPEC;
 
 	my_error = NULL;
-	addr = camel_getaddrinfo (host, service, &hints, &my_error);
+	addr = camel_getaddrinfo (
+		host, service, &hints, cancellable, &my_error);
 	if (addr == NULL && fallback_port != 0 && !g_error_matches (my_error, G_IO_ERROR, G_IO_ERROR_CANCELLED)) {
-		char str_port[16];
+		gchar str_port[16];
 
 		g_clear_error (&my_error);
 		sprintf (str_port, "%d", fallback_port);
-		addr = camel_getaddrinfo (host, str_port, &hints, &my_error);
+		addr = camel_getaddrinfo (
+			host, str_port, &hints, cancellable, &my_error);
 	}
 
 	if (addr == NULL) {
@@ -1051,15 +1139,20 @@ tcp_stream_raw_connect (CamelTcpStream *stream,
 		if (proxy_host) {
 			/* SOCKS4 only does IPv4 */
 			if (ai->ai_addr->sa_family == AF_INET)
-				priv->sockfd = connect_to_socks4_proxy (raw, proxy_host, proxy_port, ai, error);
+				priv->sockfd = connect_to_socks4_proxy (
+					raw, proxy_host, proxy_port,
+					ai, cancellable, error);
 		} else
-			priv->sockfd = socket_connect (ai, error);
+			priv->sockfd = socket_connect (ai, cancellable, error);
 
 		if (priv->sockfd) {
 			retval = 0;
 			goto out;
 		}
 
+		if (ai->ai_next != NULL)
+			g_clear_error (error); /* Only preserve the error from the last try, in case no tries are successful */
+
 		ai = ai->ai_next;
 	}
 
@@ -1109,28 +1202,28 @@ tcp_stream_raw_setsockopt (CamelTcpStream *stream,
 }
 
 static struct sockaddr *
-sockaddr_from_praddr(PRNetAddr *addr, socklen_t *len)
+sockaddr_from_praddr (PRNetAddr *addr, socklen_t *len)
 {
 	/* We assume the ip addresses are the same size - they have to be anyway */
 
 	if (addr->raw.family == PR_AF_INET) {
-		struct sockaddr_in *sin = g_malloc0(sizeof(*sin));
+		struct sockaddr_in *sin = g_malloc0 (sizeof (*sin));
 
 		sin->sin_family = AF_INET;
 		sin->sin_port = addr->inet.port;
-		memcpy(&sin->sin_addr, &addr->inet.ip, sizeof(sin->sin_addr));
+		memcpy (&sin->sin_addr, &addr->inet.ip, sizeof (sin->sin_addr));
 		*len = sizeof(*sin);
 
 		return (struct sockaddr *)sin;
 	}
 #ifdef ENABLE_IPv6
 	else if (addr->raw.family == PR_AF_INET6) {
-		struct sockaddr_in6 *sin = g_malloc0(sizeof(*sin));
+		struct sockaddr_in6 *sin = g_malloc0 (sizeof (*sin));
 
 		sin->sin6_family = AF_INET6;
 		sin->sin6_port = addr->ipv6.port;
 		sin->sin6_flowinfo = addr->ipv6.flowinfo;
-		memcpy(&sin->sin6_addr, &addr->ipv6.ip, sizeof(sin->sin6_addr));
+		memcpy (&sin->sin6_addr, &addr->ipv6.ip, sizeof (sin->sin6_addr));
 		sin->sin6_scope_id = addr->ipv6.scope_id;
 		*len = sizeof(*sin);
 
@@ -1149,10 +1242,10 @@ tcp_stream_raw_get_local_address (CamelTcpStream *stream,
 	CamelTcpStreamRawPrivate *priv = raw->priv;
 	PRNetAddr addr;
 
-	if (PR_GetSockName(priv->sockfd, &addr) != PR_SUCCESS)
+	if (PR_GetSockName (priv->sockfd, &addr) != PR_SUCCESS)
 		return NULL;
 
-	return sockaddr_from_praddr(&addr, len);
+	return sockaddr_from_praddr (&addr, len);
 }
 
 static struct sockaddr *
@@ -1163,10 +1256,10 @@ tcp_stream_raw_get_remote_address (CamelTcpStream *stream,
 	CamelTcpStreamRawPrivate *priv = raw->priv;
 	PRNetAddr addr;
 
-	if (PR_GetPeerName(priv->sockfd, &addr) != PR_SUCCESS)
+	if (PR_GetPeerName (priv->sockfd, &addr) != PR_SUCCESS)
 		return NULL;
 
-	return sockaddr_from_praddr(&addr, len);
+	return sockaddr_from_praddr (&addr, len);
 }
 
 static PRFileDesc *
