@@ -33,115 +33,50 @@
 
 #include <glib/gi18n-lib.h>
 
-#include "camel-data-cache.h"
-#include "camel-exception.h"
-#include "camel-mime-message.h"
-#include "camel-operation.h"
 #include "camel-pop3-folder.h"
 #include "camel-pop3-store.h"
-#include "camel-stream-filter.h"
-#include "camel-stream-mem.h"
 
 #define d(x)
 
-#define CF_CLASS(o) (CAMEL_FOLDER_CLASS (CAMEL_OBJECT_GET_CLASS(o)))
-static CamelFolderClass *parent_class;
-
-static void pop3_finalize (CamelObject *object);
-static void pop3_refresh_info (CamelFolder *folder, CamelException *ex);
-static void pop3_sync (CamelFolder *folder, gboolean expunge, CamelException *ex);
-static gint pop3_get_message_count (CamelFolder *folder);
-static GPtrArray *pop3_get_uids (CamelFolder *folder);
-static CamelMimeMessage *pop3_get_message (CamelFolder *folder, const gchar *uid, CamelException *ex);
-static gboolean pop3_set_message_flags (CamelFolder *folder, const gchar *uid, guint32 flags, guint32 set);
-static gchar * pop3_get_filename (CamelFolder *folder, const gchar *uid, CamelException *ex);
+G_DEFINE_TYPE (CamelPOP3Folder, camel_pop3_folder, CAMEL_TYPE_FOLDER)
 
 static void
-camel_pop3_folder_class_init (CamelPOP3FolderClass *camel_pop3_folder_class)
+cmd_uidl (CamelPOP3Engine *pe,
+          CamelPOP3Stream *stream,
+          gpointer data)
 {
-	CamelFolderClass *camel_folder_class = CAMEL_FOLDER_CLASS(camel_pop3_folder_class);
+	gint ret;
+	guint len;
+	guchar *line;
+	gchar uid[1025];
+	guint id;
+	CamelPOP3FolderInfo *fi;
+	CamelPOP3Folder *folder = data;
 
-	parent_class = CAMEL_FOLDER_CLASS(camel_folder_get_type());
-
-	/* virtual method overload */
-	camel_folder_class->refresh_info = pop3_refresh_info;
-	camel_folder_class->sync = pop3_sync;
-
-	camel_folder_class->get_message_count = pop3_get_message_count;
-	camel_folder_class->get_uids = pop3_get_uids;
-	camel_folder_class->free_uids = camel_folder_free_shallow;
-	camel_folder_class->get_filename = pop3_get_filename;
-
-	camel_folder_class->get_message = pop3_get_message;
-	camel_folder_class->set_message_flags = pop3_set_message_flags;
-}
-
-CamelType
-camel_pop3_folder_get_type (void)
-{
-	static CamelType camel_pop3_folder_type = CAMEL_INVALID_TYPE;
-
-	if (!camel_pop3_folder_type) {
-		camel_pop3_folder_type = camel_type_register (CAMEL_FOLDER_TYPE, "CamelPOP3Folder",
-							      sizeof (CamelPOP3Folder),
-							      sizeof (CamelPOP3FolderClass),
-							      (CamelObjectClassInitFunc) camel_pop3_folder_class_init,
-							      NULL,
-							      NULL,
-							      (CamelObjectFinalizeFunc) pop3_finalize);
-	}
-
-	return camel_pop3_folder_type;
-}
-
-static void
-pop3_finalize (CamelObject *object)
-{
-	CamelPOP3Folder *pop3_folder = CAMEL_POP3_FOLDER (object);
-	CamelPOP3FolderInfo **fi = (CamelPOP3FolderInfo **)pop3_folder->uids->pdata;
-	CamelPOP3Store *pop3_store = (CamelPOP3Store *)((CamelFolder *)pop3_folder)->parent_store;
-	gint i;
-
-	if (pop3_folder->uids) {
-		for (i=0;i<pop3_folder->uids->len;i++,fi++) {
-			if (fi[0]->cmd) {
-				while (camel_pop3_engine_iterate(pop3_store->engine, fi[0]->cmd) > 0)
-					;
-				camel_pop3_engine_command_free(pop3_store->engine, fi[0]->cmd);
+	do {
+		ret = camel_pop3_stream_line (stream, &line, &len);
+		if (ret>=0) {
+			if (strlen ((gchar *) line) > 1024)
+				line[1024] = 0;
+			if (sscanf((gchar *) line, "%u %s", &id, uid) == 2) {
+				fi = g_hash_table_lookup (folder->uids_id, GINT_TO_POINTER (id));
+				if (fi) {
+					camel_operation_progress (NULL, (fi->index+1) * 100 / folder->uids->len);
+					fi->uid = g_strdup (uid);
+					g_hash_table_insert (folder->uids_uid, fi->uid, fi);
+				} else {
+					g_warning("ID %u (uid: %s) not in previous LIST output", id, uid);
+				}
 			}
-
-			g_free(fi[0]->uid);
-			g_free(fi[0]);
 		}
-
-		g_ptr_array_free(pop3_folder->uids, TRUE);
-		g_hash_table_destroy(pop3_folder->uids_uid);
-	}
-}
-
-CamelFolder *
-camel_pop3_folder_new (CamelStore *parent, CamelException *ex)
-{
-	CamelFolder *folder;
-
-	d(printf("opening pop3 INBOX folder\n"));
-
-	folder = CAMEL_FOLDER (camel_object_new (CAMEL_POP3_FOLDER_TYPE));
-	camel_folder_construct (folder, parent, "inbox", "inbox");
-
-	/* mt-ok, since we dont have the folder-lock for new() */
-	camel_folder_refresh_info (folder, ex);/* mt-ok */
-	if (camel_exception_is_set (ex)) {
-		camel_object_unref (CAMEL_OBJECT (folder));
-		folder = NULL;
-	}
-
-	return folder;
+	} while (ret>0);
 }
 
 /* create a uid from md5 of 'top' output */
 static void
-cmd_builduid(CamelPOP3Engine *pe, CamelPOP3Stream *stream, gpointer data)
+cmd_builduid (CamelPOP3Engine *pe,
+              CamelPOP3Stream *stream,
+              gpointer data)
 {
 	GChecksum *checksum;
 	CamelPOP3FolderInfo *fi = data;
@@ -155,16 +90,16 @@ cmd_builduid(CamelPOP3Engine *pe, CamelPOP3Stream *stream, gpointer data)
 
 	/* TODO; somehow work out the limit and use that for proper progress reporting
 	   We need a pointer to the folder perhaps? */
-	camel_operation_progress_count(NULL, fi->id);
+	camel_operation_progress (NULL, fi->id);
 
 	checksum = g_checksum_new (G_CHECKSUM_MD5);
-	mp = camel_mime_parser_new();
-	camel_mime_parser_init_with_stream(mp, (CamelStream *)stream);
-	switch (camel_mime_parser_step(mp, NULL, NULL)) {
+	mp = camel_mime_parser_new ();
+	camel_mime_parser_init_with_stream (mp, (CamelStream *)stream, NULL);
+	switch (camel_mime_parser_step (mp, NULL, NULL)) {
 	case CAMEL_MIME_PARSER_STATE_HEADER:
 	case CAMEL_MIME_PARSER_STATE_MESSAGE:
 	case CAMEL_MIME_PARSER_STATE_MULTIPART:
-		h = camel_mime_parser_headers_raw(mp);
+		h = camel_mime_parser_headers_raw (mp);
 		while (h) {
 			if (g_ascii_strcasecmp(h->name, "status") != 0
 			    && g_ascii_strcasecmp(h->name, "x-status") != 0) {
@@ -176,7 +111,7 @@ cmd_builduid(CamelPOP3Engine *pe, CamelPOP3Stream *stream, gpointer data)
 	default:
 		break;
 	}
-	camel_object_unref(mp);
+	g_object_unref (mp);
 	g_checksum_get_digest (checksum, digest, &length);
 	g_checksum_free (checksum);
 
@@ -186,153 +121,471 @@ cmd_builduid(CamelPOP3Engine *pe, CamelPOP3Stream *stream, gpointer data)
 }
 
 static void
-cmd_list(CamelPOP3Engine *pe, CamelPOP3Stream *stream, gpointer data)
+cmd_list (CamelPOP3Engine *pe, CamelPOP3Stream *stream, gpointer data)
 {
 	gint ret;
 	guint len, id, size;
 	guchar *line;
 	CamelFolder *folder = data;
-	CamelPOP3Store *pop3_store = CAMEL_POP3_STORE (folder->parent_store);
+	CamelStore *parent_store;
+	CamelPOP3Store *pop3_store;
 	CamelPOP3FolderInfo *fi;
 
+	parent_store = camel_folder_get_parent_store (folder);
+	pop3_store = CAMEL_POP3_STORE (parent_store);
+
 	do {
-		ret = camel_pop3_stream_line(stream, &line, &len);
+		ret = camel_pop3_stream_line (stream, &line, &len);
 		if (ret>=0) {
 			if (sscanf((gchar *) line, "%u %u", &id, &size) == 2) {
-				fi = g_malloc0(sizeof(*fi));
+				fi = g_malloc0 (sizeof (*fi));
 				fi->size = size;
 				fi->id = id;
 				fi->index = ((CamelPOP3Folder *)folder)->uids->len;
 				if ((pop3_store->engine->capa & CAMEL_POP3_CAP_UIDL) == 0)
 					fi->cmd = camel_pop3_engine_command_new(pe, CAMEL_POP3_COMMAND_MULTI, cmd_builduid, fi, "TOP %u 0\r\n", id);
-				g_ptr_array_add(((CamelPOP3Folder *)folder)->uids, fi);
-				g_hash_table_insert(((CamelPOP3Folder *)folder)->uids_id, GINT_TO_POINTER(id), fi);
+				g_ptr_array_add (((CamelPOP3Folder *)folder)->uids, fi);
+				g_hash_table_insert (((CamelPOP3Folder *)folder)->uids_id, GINT_TO_POINTER (id), fi);
 			}
 		}
 	} while (ret>0);
 }
 
 static void
-cmd_uidl(CamelPOP3Engine *pe, CamelPOP3Stream *stream, gpointer data)
+cmd_tocache (CamelPOP3Engine *pe,
+             CamelPOP3Stream *stream,
+             gpointer data)
 {
-	gint ret;
-	guint len;
-	guchar *line;
-	gchar uid[1025];
-	guint id;
-	CamelPOP3FolderInfo *fi;
-	CamelPOP3Folder *folder = data;
+	CamelPOP3FolderInfo *fi = data;
+	gchar buffer[2048];
+	gint w = 0, n;
 
-	do {
-		ret = camel_pop3_stream_line(stream, &line, &len);
-		if (ret>=0) {
-			if (strlen((gchar *) line) > 1024)
-				line[1024] = 0;
-			if (sscanf((gchar *) line, "%u %s", &id, uid) == 2) {
-				fi = g_hash_table_lookup(folder->uids_id, GINT_TO_POINTER(id));
-				if (fi) {
-					camel_operation_progress(NULL, (fi->index+1) * 100 / folder->uids->len);
-					fi->uid = g_strdup(uid);
-					g_hash_table_insert(folder->uids_uid, fi->uid, fi);
-				} else {
-					g_warning("ID %u (uid: %s) not in previous LIST output", id, uid);
+	/* What if it fails? */
+
+	/* We write an '*' to the start of the stream to say its not complete yet */
+	/* This should probably be part of the cache code */
+	if ((n = camel_stream_write (fi->stream, "*", 1, NULL, NULL)) == -1)
+		goto done;
+
+	while ((n = camel_stream_read ((CamelStream *)stream, buffer, sizeof (buffer), NULL, NULL)) > 0) {
+		n = camel_stream_write (fi->stream, buffer, n, NULL, NULL);
+		if (n == -1)
+			break;
+
+		w += n;
+		if (w > fi->size)
+			w = fi->size;
+		if (fi->size != 0)
+			camel_operation_progress (NULL, (w * 100) / fi->size);
+	}
+
+	/* it all worked, output a '#' to say we're a-ok */
+	if (n != -1) {
+		camel_stream_reset (fi->stream, NULL);
+		n = camel_stream_write(fi->stream, "#", 1, NULL, NULL);
+	}
+done:
+	if (n == -1) {
+		fi->err = errno;
+		g_warning("POP3 retrieval failed: %s", g_strerror(errno));
+	} else {
+		fi->err = 0;
+	}
+
+	g_object_unref (fi->stream);
+	fi->stream = NULL;
+}
+
+static void
+pop3_folder_dispose (GObject *object)
+{
+	CamelPOP3Folder *pop3_folder = CAMEL_POP3_FOLDER (object);
+	CamelPOP3FolderInfo **fi = (CamelPOP3FolderInfo **)pop3_folder->uids->pdata;
+	CamelPOP3Store *pop3_store;
+	CamelStore *parent_store;
+	gint i;
+
+	parent_store = camel_folder_get_parent_store (CAMEL_FOLDER (object));
+	if (parent_store) {
+		pop3_store = CAMEL_POP3_STORE (parent_store);
+
+		if (pop3_folder->uids) {
+			for (i = 0; i < pop3_folder->uids->len; i++, fi++) {
+				if (fi[0]->cmd) {
+					while (camel_pop3_engine_iterate (pop3_store->engine, fi[0]->cmd) > 0)
+						;
+					camel_pop3_engine_command_free (pop3_store->engine, fi[0]->cmd);
+				}
+
+				g_free (fi[0]->uid);
+				g_free (fi[0]);
+			}
+
+			g_ptr_array_free (pop3_folder->uids, TRUE);
+			g_hash_table_destroy (pop3_folder->uids_uid);
+		}
+	}
+
+	/* Chain up to parent's dispose() method. */
+	G_OBJECT_CLASS (camel_pop3_folder_parent_class)->dispose (object);
+}
+
+static gint
+pop3_folder_get_message_count (CamelFolder *folder)
+{
+	CamelPOP3Folder *pop3_folder = CAMEL_POP3_FOLDER (folder);
+
+	return pop3_folder->uids->len;
+}
+
+static GPtrArray *
+pop3_folder_get_uids (CamelFolder *folder)
+{
+	CamelPOP3Folder *pop3_folder = CAMEL_POP3_FOLDER (folder);
+	GPtrArray *uids = g_ptr_array_new ();
+	CamelPOP3FolderInfo **fi = (CamelPOP3FolderInfo **)pop3_folder->uids->pdata;
+	gint i;
+
+	for (i=0;i<pop3_folder->uids->len;i++,fi++) {
+		if (fi[0]->uid)
+			g_ptr_array_add (uids, fi[0]->uid);
+	}
+
+	return uids;
+}
+
+static gchar *
+pop3_folder_get_filename (CamelFolder *folder,
+                          const gchar *uid,
+                          GError **error)
+{
+	CamelStore *parent_store;
+	CamelPOP3Folder *pop3_folder;
+	CamelPOP3Store *pop3_store;
+	CamelPOP3FolderInfo *fi;
+
+	parent_store = camel_folder_get_parent_store (folder);
+
+	pop3_folder = CAMEL_POP3_FOLDER (folder);
+	pop3_store = CAMEL_POP3_STORE (parent_store);
+
+	fi = g_hash_table_lookup (pop3_folder->uids_uid, uid);
+	if (fi == NULL) {
+		g_set_error (
+			error, CAMEL_FOLDER_ERROR,
+			CAMEL_FOLDER_ERROR_INVALID_UID,
+			_("No message with UID %s"), uid);
+		return NULL;
+	}
+
+	return camel_data_cache_get_filename (
+		pop3_store->cache, "cache", fi->uid, NULL);
+}
+
+static gboolean
+pop3_folder_set_message_flags (CamelFolder *folder,
+                               const gchar *uid,
+                               CamelMessageFlags flags,
+                               CamelMessageFlags set)
+{
+	CamelPOP3Folder *pop3_folder = CAMEL_POP3_FOLDER (folder);
+	CamelPOP3FolderInfo *fi;
+	gboolean res = FALSE;
+
+	fi = g_hash_table_lookup (pop3_folder->uids_uid, uid);
+	if (fi) {
+		guint32 new = (fi->flags & ~flags) | (set & flags);
+
+		if (fi->flags != new) {
+			fi->flags = new;
+			res = TRUE;
+		}
+	}
+
+	return res;
+}
+
+static CamelMimeMessage *
+pop3_folder_get_message_sync (CamelFolder *folder,
+                              const gchar *uid,
+                              GCancellable *cancellable,
+                              GError **error)
+{
+	CamelStore *parent_store;
+	CamelMimeMessage *message = NULL;
+	CamelPOP3Store *pop3_store;
+	CamelPOP3Folder *pop3_folder;
+	CamelPOP3Command *pcr;
+	CamelPOP3FolderInfo *fi;
+	gchar buffer[1];
+	gint i, last;
+	CamelStream *stream = NULL;
+
+	parent_store = camel_folder_get_parent_store (folder);
+
+	pop3_folder = CAMEL_POP3_FOLDER (folder);
+	pop3_store = CAMEL_POP3_STORE (parent_store);
+
+	fi = g_hash_table_lookup (pop3_folder->uids_uid, uid);
+	if (fi == NULL) {
+		g_set_error (
+			error, CAMEL_FOLDER_ERROR,
+			CAMEL_FOLDER_ERROR_INVALID_UID,
+			_("No message with UID %s"), uid);
+		return NULL;
+	}
+
+	/* Sigh, most of the crap in this function is so that the cancel button
+	   returns the proper exception code.  Sigh. */
+
+	camel_operation_push_message (
+		cancellable, _("Retrieving POP message %d"), fi->id);
+
+	/* If we have an oustanding retrieve message running, wait for that to complete
+	   & then retrieve from cache, otherwise, start a new one, and similar */
+
+	if (fi->cmd != NULL) {
+		while ((i = camel_pop3_engine_iterate (pop3_store->engine, fi->cmd)) > 0)
+			;
+
+		if (i == -1)
+			fi->err = errno;
+
+		/* getting error code? */
+		/*g_assert (fi->cmd->state == CAMEL_POP3_COMMAND_DATA);*/
+		camel_pop3_engine_command_free (pop3_store->engine, fi->cmd);
+		fi->cmd = NULL;
+
+		if (fi->err != 0) {
+			if (fi->err == EINTR)
+				g_set_error (
+					error, G_IO_ERROR,
+					G_IO_ERROR_CANCELLED,
+					_("Cancelled"));
+			else
+				g_set_error (
+					error, CAMEL_ERROR,
+					CAMEL_ERROR_GENERIC,
+					_("Cannot get message %s: %s"),
+					uid, g_strerror (fi->err));
+			goto fail;
+		}
+	}
+
+	/* check to see if we have safely written flag set */
+	if (pop3_store->cache == NULL
+	    || (stream = camel_data_cache_get(pop3_store->cache, "cache", fi->uid, NULL)) == NULL
+	    || camel_stream_read (stream, buffer, 1, cancellable, NULL) != 1
+	    || buffer[0] != '#') {
+
+		/* Initiate retrieval, if disk backing fails, use a memory backing */
+		if (pop3_store->cache == NULL
+		    || (stream = camel_data_cache_add(pop3_store->cache, "cache", fi->uid, NULL)) == NULL)
+			stream = camel_stream_mem_new ();
+
+		/* ref it, the cache storage routine unref's when done */
+		fi->stream = g_object_ref (stream);
+		fi->err = EIO;
+		pcr = camel_pop3_engine_command_new(pop3_store->engine, CAMEL_POP3_COMMAND_MULTI, cmd_tocache, fi, "RETR %u\r\n", fi->id);
+
+		/* Also initiate retrieval of some of the following messages, assume we'll be receiving them */
+		if (pop3_store->cache != NULL) {
+			/* This should keep track of the last one retrieved, also how many are still
+			   oustanding incase of random access on large folders */
+			i = fi->index+1;
+			last = MIN (i+10, pop3_folder->uids->len);
+			for (;i<last;i++) {
+				CamelPOP3FolderInfo *pfi = pop3_folder->uids->pdata[i];
+
+				if (pfi->uid && pfi->cmd == NULL) {
+					pfi->stream = camel_data_cache_add(pop3_store->cache, "cache", pfi->uid, NULL);
+					if (pfi->stream) {
+						pfi->err = EIO;
+						pfi->cmd = camel_pop3_engine_command_new (pop3_store->engine, CAMEL_POP3_COMMAND_MULTI,
+											 cmd_tocache, pfi, "RETR %u\r\n", pfi->id);
+					}
 				}
 			}
 		}
-	} while (ret>0);
+
+		/* now wait for the first one to finish */
+		while ((i = camel_pop3_engine_iterate (pop3_store->engine, pcr)) > 0)
+			;
+
+		if (i == -1)
+			fi->err = errno;
+
+		/* getting error code? */
+		/*g_assert (pcr->state == CAMEL_POP3_COMMAND_DATA);*/
+		camel_pop3_engine_command_free (pop3_store->engine, pcr);
+		camel_stream_reset (stream, NULL);
+
+		/* Check to see we have safely written flag set */
+		if (fi->err != 0) {
+			if (fi->err == EINTR)
+				g_set_error (
+					error, G_IO_ERROR,
+					G_IO_ERROR_CANCELLED,
+					_("Cancelled"));
+			else
+				g_set_error (
+					error, CAMEL_ERROR,
+					CAMEL_ERROR_GENERIC,
+					_("Cannot get message %s: %s"),
+					uid, g_strerror (fi->err));
+			goto done;
+		}
+
+		if (camel_stream_read (
+			stream, buffer, 1, cancellable, error) == -1)
+			goto done;
+
+		if (buffer[0] != '#') {
+			g_set_error (
+				error, CAMEL_ERROR, CAMEL_ERROR_GENERIC,
+				_("Cannot get message %s: %s"), uid,
+				_("Unknown reason"));
+			goto done;
+		}
+	}
+
+	message = camel_mime_message_new ();
+	if (!camel_data_wrapper_construct_from_stream_sync (
+		CAMEL_DATA_WRAPPER (message), stream, cancellable, error)) {
+		g_prefix_error (error, _("Cannot get message %s: "), uid);
+		g_object_unref (message);
+		message = NULL;
+	} else {
+		/* because the UID in the local store doesn't match with the UID in the pop3 store */
+		camel_medium_add_header (CAMEL_MEDIUM (message), "X-Evolution-POP3-UID", uid);
+	}
+done:
+	g_object_unref (stream);
+fail:
+	camel_operation_pop_message (cancellable);
+
+	return message;
 }
 
-static void
-pop3_refresh_info (CamelFolder *folder, CamelException *ex)
+static gboolean
+pop3_folder_refresh_info_sync (CamelFolder *folder,
+                               GCancellable *cancellable,
+                               GError **error)
 {
-	CamelPOP3Store *pop3_store = CAMEL_POP3_STORE (folder->parent_store);
+	CamelStore *parent_store;
+	CamelPOP3Store *pop3_store;
 	CamelPOP3Folder *pop3_folder = (CamelPOP3Folder *) folder;
 	CamelPOP3Command *pcl, *pcu = NULL;
+	gboolean success = TRUE;
 	gint i;
 
-	camel_operation_start (NULL, _("Retrieving POP summary"));
+	parent_store = camel_folder_get_parent_store (folder);
+	pop3_store = CAMEL_POP3_STORE (parent_store);
+
+	camel_operation_push_message (
+		cancellable, _("Retrieving POP summary"));
 
 	pop3_folder->uids = g_ptr_array_new ();
-	pop3_folder->uids_uid = g_hash_table_new(g_str_hash, g_str_equal);
+	pop3_folder->uids_uid = g_hash_table_new (g_str_hash, g_str_equal);
 	/* only used during setup */
-	pop3_folder->uids_id = g_hash_table_new(NULL, NULL);
+	pop3_folder->uids_id = g_hash_table_new (NULL, NULL);
 
 	pcl = camel_pop3_engine_command_new(pop3_store->engine, CAMEL_POP3_COMMAND_MULTI, cmd_list, folder, "LIST\r\n");
 	if (pop3_store->engine->capa & CAMEL_POP3_CAP_UIDL)
 		pcu = camel_pop3_engine_command_new(pop3_store->engine, CAMEL_POP3_COMMAND_MULTI, cmd_uidl, folder, "UIDL\r\n");
-	while ((i = camel_pop3_engine_iterate(pop3_store->engine, NULL)) > 0)
+	while ((i = camel_pop3_engine_iterate (pop3_store->engine, NULL)) > 0)
 		;
 
 	if (i == -1) {
 		if (errno == EINTR)
-			camel_exception_setv(ex, CAMEL_EXCEPTION_USER_CANCEL, _("User canceled"));
+			g_set_error (
+				error, G_IO_ERROR,
+				G_IO_ERROR_CANCELLED,
+				_("Cancelled"));
 		else
-			camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
-					      _("Cannot get POP summary: %s"),
-					      g_strerror (errno));
+			g_set_error (
+				error, G_IO_ERROR,
+				g_io_error_from_errno (errno),
+				_("Cannot get POP summary: %s"),
+				g_strerror (errno));
+		success = FALSE;
 	}
 
 	/* TODO: check every id has a uid & commands returned OK too? */
 
-	camel_pop3_engine_command_free(pop3_store->engine, pcl);
+	camel_pop3_engine_command_free (pop3_store->engine, pcl);
 
 	if (pop3_store->engine->capa & CAMEL_POP3_CAP_UIDL) {
-		camel_pop3_engine_command_free(pop3_store->engine, pcu);
+		camel_pop3_engine_command_free (pop3_store->engine, pcu);
 	} else {
 		for (i=0;i<pop3_folder->uids->len;i++) {
 			CamelPOP3FolderInfo *fi = pop3_folder->uids->pdata[i];
 			if (fi->cmd) {
-				camel_pop3_engine_command_free(pop3_store->engine, fi->cmd);
+				camel_pop3_engine_command_free (pop3_store->engine, fi->cmd);
 				fi->cmd = NULL;
 			}
 			if (fi->uid)
-				g_hash_table_insert(pop3_folder->uids_uid, fi->uid, fi);
+				g_hash_table_insert (pop3_folder->uids_uid, fi->uid, fi);
 		}
 	}
 
 	/* dont need this anymore */
-	g_hash_table_destroy(pop3_folder->uids_id);
+	g_hash_table_destroy (pop3_folder->uids_id);
 
-	camel_operation_end (NULL);
-	return;
+	camel_operation_pop_message (cancellable);
+
+	return success;
 }
 
-static void
-pop3_sync (CamelFolder *folder, gboolean expunge, CamelException *ex)
+static gboolean
+pop3_folder_synchronize_sync (CamelFolder *folder,
+                              gboolean expunge,
+                              GCancellable *cancellable,
+                              GError **error)
 {
+	CamelStore *parent_store;
 	CamelPOP3Folder *pop3_folder;
 	CamelPOP3Store *pop3_store;
 	gint i;
 	CamelPOP3FolderInfo *fi;
 
-	pop3_folder = CAMEL_POP3_FOLDER (folder);
-	pop3_store = CAMEL_POP3_STORE (folder->parent_store);
+	parent_store = camel_folder_get_parent_store (folder);
 
-	if (pop3_store->delete_after && !expunge) {
+	pop3_folder = CAMEL_POP3_FOLDER (folder);
+	pop3_store = CAMEL_POP3_STORE (parent_store);
+
+	if (pop3_store->delete_after > 0 && !expunge) {
 		d(printf("%s(%d): pop3_store->delete_after = [%d], expunge=[%d]\n",
 			 __FILE__, __LINE__, pop3_store->delete_after, expunge));
-		camel_operation_start(NULL, _("Expunging old messages"));
-		camel_pop3_delete_old(folder, pop3_store->delete_after,ex);
+		camel_operation_push_message (
+			cancellable, _("Expunging old messages"));
+
+		camel_pop3_delete_old (
+			folder, pop3_store->delete_after,
+			cancellable, error);
+
+		camel_operation_pop_message (cancellable);
 	}
 
-	if (!expunge) {
-		return;
+	if (!expunge || (pop3_store->keep_on_server && !pop3_store->delete_expunged)) {
+		return TRUE;
 	}
 
-	camel_operation_start(NULL, _("Expunging deleted messages"));
+	camel_operation_push_message (
+		cancellable, _("Expunging deleted messages"));
 
 	for (i = 0; i < pop3_folder->uids->len; i++) {
 		fi = pop3_folder->uids->pdata[i];
 		/* busy already?  wait for that to finish first */
 		if (fi->cmd) {
-			while (camel_pop3_engine_iterate(pop3_store->engine, fi->cmd) > 0)
+			while (camel_pop3_engine_iterate (pop3_store->engine, fi->cmd) > 0)
 				;
-			camel_pop3_engine_command_free(pop3_store->engine, fi->cmd);
+			camel_pop3_engine_command_free (pop3_store->engine, fi->cmd);
 			fi->cmd = NULL;
 		}
 
 		if (fi->flags & CAMEL_MESSAGE_DELETED) {
-			fi->cmd = camel_pop3_engine_command_new(pop3_store->engine,
+			fi->cmd = camel_pop3_engine_command_new (pop3_store->engine,
 								0,
 								NULL,
 								NULL,
@@ -349,22 +602,74 @@ pop3_sync (CamelFolder *folder, gboolean expunge, CamelException *ex)
 		fi = pop3_folder->uids->pdata[i];
 		/* wait for delete commands to finish */
 		if (fi->cmd) {
-			while (camel_pop3_engine_iterate(pop3_store->engine, fi->cmd) > 0)
+			while (camel_pop3_engine_iterate (pop3_store->engine, fi->cmd) > 0)
 				;
-			camel_pop3_engine_command_free(pop3_store->engine, fi->cmd);
+			camel_pop3_engine_command_free (pop3_store->engine, fi->cmd);
 			fi->cmd = NULL;
 		}
-		camel_operation_progress(NULL, (i+1) * 100 / pop3_folder->uids->len);
+		camel_operation_progress (
+			cancellable, (i+1) * 100 / pop3_folder->uids->len);
 	}
 
-	camel_operation_end(NULL);
+	camel_operation_pop_message (cancellable);
 
-	camel_pop3_store_expunge (pop3_store, ex);
+	camel_pop3_store_expunge (pop3_store, error);
+
+	return TRUE;
+}
+
+static void
+camel_pop3_folder_class_init (CamelPOP3FolderClass *class)
+{
+	GObjectClass *object_class;
+	CamelFolderClass *folder_class;
+
+	object_class = G_OBJECT_CLASS (class);
+	object_class->dispose = pop3_folder_dispose;
+
+	folder_class = CAMEL_FOLDER_CLASS (class);
+	folder_class->get_message_count = pop3_folder_get_message_count;
+	folder_class->get_uids = pop3_folder_get_uids;
+	folder_class->free_uids = camel_folder_free_shallow;
+	folder_class->get_filename = pop3_folder_get_filename;
+	folder_class->set_message_flags = pop3_folder_set_message_flags;
+	folder_class->get_message_sync = pop3_folder_get_message_sync;
+	folder_class->refresh_info_sync = pop3_folder_refresh_info_sync;
+	folder_class->synchronize_sync = pop3_folder_synchronize_sync;
+}
+
+static void
+camel_pop3_folder_init (CamelPOP3Folder *pop3_folder)
+{
+}
+
+CamelFolder *
+camel_pop3_folder_new (CamelStore *parent,
+                       GCancellable *cancellable,
+                       GError **error)
+{
+	CamelFolder *folder;
+
+	d(printf("opening pop3 INBOX folder\n"));
+
+	folder = g_object_new (
+		CAMEL_TYPE_POP3_FOLDER,
+		"full-name", "inbox", "name", "inbox",
+		"parent-store", parent, NULL);
+
+	/* mt-ok, since we dont have the folder-lock for new() */
+	if (!camel_folder_refresh_info_sync (folder, cancellable, error)) {
+		g_object_unref (folder);
+		folder = NULL;
+	}
+
+	return folder;
 }
 
 static gboolean
 pop3_get_message_time_from_cache (CamelFolder *folder, const gchar *uid, time_t *message_time)
 {
+	CamelStore *parent_store;
 	CamelPOP3Store *pop3_store;
 	CamelStream *stream = NULL;
 	gchar buffer[1];
@@ -374,19 +679,21 @@ pop3_get_message_time_from_cache (CamelFolder *folder, const gchar *uid, time_t
 	g_return_val_if_fail (uid != NULL, FALSE);
 	g_return_val_if_fail (message_time != NULL, FALSE);
 
-	pop3_store = CAMEL_POP3_STORE (folder->parent_store);
+	parent_store = camel_folder_get_parent_store (folder);
+	pop3_store = CAMEL_POP3_STORE (parent_store);
 
 	g_return_val_if_fail (pop3_store->cache != NULL, FALSE);
 
 	if ((stream = camel_data_cache_get (pop3_store->cache, "cache", uid, NULL)) != NULL
-	    && camel_stream_read (stream, buffer, 1) == 1
+	    && camel_stream_read (stream, buffer, 1, NULL, NULL) == 1
 	    && buffer[0] == '#') {
 		CamelMimeMessage *message;
 
 		message = camel_mime_message_new ();
-		if (camel_data_wrapper_construct_from_stream ((CamelDataWrapper *)message, stream) == -1) {
+		if (!camel_data_wrapper_construct_from_stream_sync (
+			(CamelDataWrapper *)message, stream, NULL, NULL)) {
 			g_warning (_("Cannot get message %s: %s"), uid, g_strerror (errno));
-			camel_object_unref ((CamelObject *)message);
+			g_object_unref (message);
 			message = NULL;
 		}
 
@@ -394,54 +701,70 @@ pop3_get_message_time_from_cache (CamelFolder *folder, const gchar *uid, time_t
 			res = TRUE;
 			*message_time = message->date + message->date_offset;
 
-			camel_object_unref ((CamelObject *)message);
+			g_object_unref (message);
 		}
 	}
 
 	if (stream) {
-		camel_object_unref (stream);
+		g_object_unref (stream);
 	}
 	return res;
 }
 
 gint
-camel_pop3_delete_old(CamelFolder *folder, gint days_to_delete,	CamelException *ex)
+camel_pop3_delete_old (CamelFolder *folder,
+                       gint days_to_delete,
+                       GCancellable *cancellable,
+                       GError **error)
 {
+	CamelStore *parent_store;
 	CamelPOP3Folder *pop3_folder;
 	CamelPOP3FolderInfo *fi;
 	gint i;
 	CamelPOP3Store *pop3_store;
-	time_t temp, message_time;
+	CamelMimeMessage *message;
+	time_t temp, message_time = 0;
+
+	parent_store = camel_folder_get_parent_store (folder);
 
 	pop3_folder = CAMEL_POP3_FOLDER (folder);
-	pop3_store = CAMEL_POP3_STORE (CAMEL_FOLDER(pop3_folder)->parent_store);
-	temp = time(&temp);
+	pop3_store = CAMEL_POP3_STORE (parent_store);
+	temp = time (&temp);
 
 	d(printf("%s(%d): pop3_folder->uids->len=[%d]\n", __FILE__, __LINE__, pop3_folder->uids->len));
 	for (i = 0; i < pop3_folder->uids->len; i++) {
 		fi = pop3_folder->uids->pdata[i];
 
 		d(printf("%s(%d): fi->uid=[%s]\n", __FILE__, __LINE__, fi->uid));
-		if (pop3_get_message_time_from_cache (folder, fi->uid, &message_time)) {
-			gdouble time_diff = difftime(temp,message_time);
+		if (!pop3_get_message_time_from_cache (folder, fi->uid, &message_time)) {
+			d(printf("could not get message time from cache, trying from pop3\n"));
+			message = pop3_folder_get_message_sync (
+				folder, fi->uid, cancellable, error);
+			if (message) {
+				message_time = message->date + message->date_offset;
+				g_object_unref (message);
+			}
+		}
+
+		if (message_time) {
+			gdouble time_diff = difftime (temp,message_time);
 			gint day_lag = time_diff/(60*60*24);
 
 			d(printf("%s(%d): message_time= [%ld]\n", __FILE__, __LINE__, message_time));
 			d(printf("%s(%d): day_lag=[%d] \t days_to_delete=[%d]\n",
-				 __FILE__, __LINE__, day_lag, days_to_delete));
+				__FILE__, __LINE__, day_lag, days_to_delete));
 
-			if ( day_lag > days_to_delete)
-			{
+			if (day_lag > days_to_delete) {
 				if (fi->cmd) {
-					while (camel_pop3_engine_iterate(pop3_store->engine, fi->cmd) > 0) {
+					while (camel_pop3_engine_iterate (pop3_store->engine, fi->cmd) > 0) {
 						; /* do nothing - iterating until end */
 					}
 
-					camel_pop3_engine_command_free(pop3_store->engine, fi->cmd);
+					camel_pop3_engine_command_free (pop3_store->engine, fi->cmd);
 					fi->cmd = NULL;
 				}
 				d(printf("%s(%d): Deleting old messages\n", __FILE__, __LINE__));
-				fi->cmd = camel_pop3_engine_command_new(pop3_store->engine,
+				fi->cmd = camel_pop3_engine_command_new (pop3_store->engine,
 									0,
 									NULL,
 									NULL,
@@ -459,257 +782,16 @@ camel_pop3_delete_old(CamelFolder *folder, gint days_to_delete,	CamelException *
 		fi = pop3_folder->uids->pdata[i];
 		/* wait for delete commands to finish */
 		if (fi->cmd) {
-			while (camel_pop3_engine_iterate(pop3_store->engine, fi->cmd) > 0)
+			while (camel_pop3_engine_iterate (pop3_store->engine, fi->cmd) > 0)
 				;
-			camel_pop3_engine_command_free(pop3_store->engine, fi->cmd);
+			camel_pop3_engine_command_free (pop3_store->engine, fi->cmd);
 			fi->cmd = NULL;
 		}
-		camel_operation_progress(NULL, (i+1) * 100 / pop3_folder->uids->len);
+		camel_operation_progress (
+			cancellable, (i+1) * 100 / pop3_folder->uids->len);
 	}
 
-	camel_operation_end(NULL);
-
-	camel_pop3_store_expunge (pop3_store, ex);
+	camel_pop3_store_expunge (pop3_store, error);
 
 	return 0;
 }
-
-static void
-cmd_tocache(CamelPOP3Engine *pe, CamelPOP3Stream *stream, gpointer data)
-{
-	CamelPOP3FolderInfo *fi = data;
-	gchar buffer[2048];
-	gint w = 0, n;
-
-	/* What if it fails? */
-
-	/* We write an '*' to the start of the stream to say its not complete yet */
-	/* This should probably be part of the cache code */
-	if ((n = camel_stream_write(fi->stream, "*", 1)) == -1)
-		goto done;
-
-	while ((n = camel_stream_read((CamelStream *)stream, buffer, sizeof(buffer))) > 0) {
-		n = camel_stream_write(fi->stream, buffer, n);
-		if (n == -1)
-			break;
-
-		w += n;
-		if (w > fi->size)
-			w = fi->size;
-		if (fi->size != 0)
-			camel_operation_progress(NULL, (w * 100) / fi->size);
-	}
-
-	/* it all worked, output a '#' to say we're a-ok */
-	if (n != -1) {
-		camel_stream_reset(fi->stream);
-		n = camel_stream_write(fi->stream, "#", 1);
-	}
-done:
-	if (n == -1) {
-		fi->err = errno;
-		g_warning("POP3 retrieval failed: %s", g_strerror(errno));
-	} else {
-		fi->err = 0;
-	}
-
-	camel_object_unref((CamelObject *)fi->stream);
-	fi->stream = NULL;
-}
-
-static gchar *
-pop3_get_filename (CamelFolder *folder, const gchar *uid, CamelException *ex)
-{
-	CamelPOP3Folder *pop3_folder = (CamelPOP3Folder *)folder;
-	CamelPOP3Store *pop3_store = (CamelPOP3Store *)((CamelFolder *)pop3_folder)->parent_store;
-	CamelPOP3FolderInfo *fi;
-
-	fi = g_hash_table_lookup(pop3_folder->uids_uid, uid);
-	if (fi == NULL) {
-		camel_exception_setv (ex, CAMEL_EXCEPTION_FOLDER_INVALID_UID,
-				      _("No message with UID %s"), uid);
-		return NULL;
-	}
-
-	return camel_data_cache_get_filename (pop3_store->cache, "cache", fi->uid, NULL);
-}
-
-static CamelMimeMessage *
-pop3_get_message (CamelFolder *folder, const gchar *uid, CamelException *ex)
-{
-	CamelMimeMessage *message = NULL;
-	CamelPOP3Store *pop3_store = CAMEL_POP3_STORE (folder->parent_store);
-	CamelPOP3Folder *pop3_folder = (CamelPOP3Folder *)folder;
-	CamelPOP3Command *pcr;
-	CamelPOP3FolderInfo *fi;
-	gchar buffer[1];
-	gint i, last;
-	CamelStream *stream = NULL;
-
-	fi = g_hash_table_lookup(pop3_folder->uids_uid, uid);
-	if (fi == NULL) {
-		camel_exception_setv (ex, CAMEL_EXCEPTION_FOLDER_INVALID_UID,
-				      _("No message with UID %s"), uid);
-		return NULL;
-	}
-
-	/* Sigh, most of the crap in this function is so that the cancel button
-	   returns the proper exception code.  Sigh. */
-
-	camel_operation_start_transient(NULL, _("Retrieving POP message %d"), fi->id);
-
-	/* If we have an oustanding retrieve message running, wait for that to complete
-	   & then retrieve from cache, otherwise, start a new one, and similar */
-
-	if (fi->cmd != NULL) {
-		while ((i = camel_pop3_engine_iterate(pop3_store->engine, fi->cmd)) > 0)
-			;
-
-		if (i == -1)
-			fi->err = errno;
-
-		/* getting error code? */
-		/*g_assert (fi->cmd->state == CAMEL_POP3_COMMAND_DATA);*/
-		camel_pop3_engine_command_free(pop3_store->engine, fi->cmd);
-		fi->cmd = NULL;
-
-		if (fi->err != 0) {
-			if (fi->err == EINTR)
-				camel_exception_setv(ex, CAMEL_EXCEPTION_USER_CANCEL, _("User canceled"));
-			else
-				camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
-						      _("Cannot get message %s: %s"),
-						      uid, g_strerror (fi->err));
-			goto fail;
-		}
-	}
-
-	/* check to see if we have safely written flag set */
-	if (pop3_store->cache == NULL
-	    || (stream = camel_data_cache_get(pop3_store->cache, "cache", fi->uid, NULL)) == NULL
-	    || camel_stream_read(stream, buffer, 1) != 1
-	    || buffer[0] != '#') {
-
-		/* Initiate retrieval, if disk backing fails, use a memory backing */
-		if (pop3_store->cache == NULL
-		    || (stream = camel_data_cache_add(pop3_store->cache, "cache", fi->uid, NULL)) == NULL)
-			stream = camel_stream_mem_new();
-
-		/* ref it, the cache storage routine unref's when done */
-		camel_object_ref((CamelObject *)stream);
-		fi->stream = stream;
-		fi->err = EIO;
-		pcr = camel_pop3_engine_command_new(pop3_store->engine, CAMEL_POP3_COMMAND_MULTI, cmd_tocache, fi, "RETR %u\r\n", fi->id);
-
-		/* Also initiate retrieval of some of the following messages, assume we'll be receiving them */
-		if (pop3_store->cache != NULL) {
-			/* This should keep track of the last one retrieved, also how many are still
-			   oustanding incase of random access on large folders */
-			i = fi->index+1;
-			last = MIN(i+10, pop3_folder->uids->len);
-			for (;i<last;i++) {
-				CamelPOP3FolderInfo *pfi = pop3_folder->uids->pdata[i];
-
-				if (pfi->uid && pfi->cmd == NULL) {
-					pfi->stream = camel_data_cache_add(pop3_store->cache, "cache", pfi->uid, NULL);
-					if (pfi->stream) {
-						pfi->err = EIO;
-						pfi->cmd = camel_pop3_engine_command_new(pop3_store->engine, CAMEL_POP3_COMMAND_MULTI,
-											 cmd_tocache, pfi, "RETR %u\r\n", pfi->id);
-					}
-				}
-			}
-		}
-
-		/* now wait for the first one to finish */
-		while ((i = camel_pop3_engine_iterate(pop3_store->engine, pcr)) > 0)
-			;
-
-		if (i == -1)
-			fi->err = errno;
-
-		/* getting error code? */
-		/*g_assert (pcr->state == CAMEL_POP3_COMMAND_DATA);*/
-		camel_pop3_engine_command_free(pop3_store->engine, pcr);
-		camel_stream_reset(stream);
-
-		/* Check to see we have safely written flag set */
-		if (fi->err != 0) {
-			if (fi->err == EINTR)
-				camel_exception_setv(ex, CAMEL_EXCEPTION_USER_CANCEL, _("User canceled"));
-			else
-				camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
-					_("Cannot get message %s: %s"),
-					uid, g_strerror (fi->err));
-			goto done;
-		}
-
-		if (camel_stream_read(stream, buffer, 1) != 1 || buffer[0] != '#') {
-			camel_exception_setv(ex, CAMEL_EXCEPTION_SYSTEM,
-					_("Cannot get message %s: %s"), uid, _("Unknown reason"));
-			goto done;
-		}
-	}
-
-	message = camel_mime_message_new ();
-	if (camel_data_wrapper_construct_from_stream((CamelDataWrapper *)message, stream) == -1) {
-		if (errno == EINTR)
-			camel_exception_setv(ex, CAMEL_EXCEPTION_USER_CANCEL, _("User canceled"));
-		else
-			camel_exception_setv (ex, CAMEL_EXCEPTION_SYSTEM,
-				_("Cannot get message %s: %s"),
-				uid, g_strerror (errno));
-		camel_object_unref((CamelObject *)message);
-		message = NULL;
-	}
-done:
-	camel_object_unref((CamelObject *)stream);
-fail:
-	camel_operation_end(NULL);
-
-	return message;
-}
-
-static gboolean
-pop3_set_message_flags (CamelFolder *folder, const gchar *uid, guint32 flags, guint32 set)
-{
-	CamelPOP3Folder *pop3_folder = CAMEL_POP3_FOLDER (folder);
-	CamelPOP3FolderInfo *fi;
-	gboolean res = FALSE;
-
-	fi = g_hash_table_lookup(pop3_folder->uids_uid, uid);
-	if (fi) {
-		guint32 new = (fi->flags & ~flags) | (set & flags);
-
-		if (fi->flags != new) {
-			fi->flags = new;
-			res = TRUE;
-		}
-	}
-
-	return res;
-}
-
-static gint
-pop3_get_message_count (CamelFolder *folder)
-{
-	CamelPOP3Folder *pop3_folder = CAMEL_POP3_FOLDER (folder);
-
-	return pop3_folder->uids->len;
-}
-
-static GPtrArray *
-pop3_get_uids (CamelFolder *folder)
-{
-	CamelPOP3Folder *pop3_folder = CAMEL_POP3_FOLDER (folder);
-	GPtrArray *uids = g_ptr_array_new();
-	CamelPOP3FolderInfo **fi = (CamelPOP3FolderInfo **)pop3_folder->uids->pdata;
-	gint i;
-
-	for (i=0;i<pop3_folder->uids->len;i++,fi++) {
-		if (fi[0]->uid)
-			g_ptr_array_add(uids, fi[0]->uid);
-	}
-
-	return uids;
-}
