@@ -103,8 +103,11 @@ get_valid_utf8_error (const gchar *text)
 
 static gboolean
 connect_to_server (CamelService *service,
-		   const gchar *host, const gchar *serv, gint fallback_port,
+                   const gchar *host,
+                   const gchar *serv,
+                   gint fallback_port,
                    gint ssl_mode,
+                   GCancellable *cancellable,
                    GError **error)
 {
 	CamelPOP3Store *store = CAMEL_POP3_STORE (service);
@@ -116,7 +119,7 @@ connect_to_server (CamelService *service,
 	guint32 flags = 0;
 	gint clean_quit = TRUE;
 	gint ret;
-	const gchar *delete_days;
+	const gchar *param;
 
 	if (ssl_mode != MODE_CLEAR) {
 #ifdef CAMEL_HAVE_SSL
@@ -141,17 +144,22 @@ connect_to_server (CamelService *service,
 	camel_session_get_socks_proxy (session, &socks_host, &socks_port);
 
 	if (socks_host) {
-		camel_tcp_stream_set_socks_proxy ((CamelTcpStream *) tcp_stream, socks_host, socks_port);
+		camel_tcp_stream_set_socks_proxy (
+			CAMEL_TCP_STREAM (tcp_stream),
+			socks_host, socks_port);
 		g_free (socks_host);
 	}
 
-	if (camel_tcp_stream_connect ((CamelTcpStream *) tcp_stream, host, serv, fallback_port, error) == -1) {
+	if (camel_tcp_stream_connect (
+		CAMEL_TCP_STREAM (tcp_stream), host, serv,
+		fallback_port, cancellable, error) == -1) {
 		g_object_unref (tcp_stream);
 		return FALSE;
 	}
 
 	/* parent class connect initialization */
-	if (CAMEL_SERVICE_CLASS (camel_pop3_store_parent_class)->connect (service, error) == FALSE) {
+	if (CAMEL_SERVICE_CLASS (camel_pop3_store_parent_class)->
+		connect_sync (service, cancellable, error) == FALSE) {
 		g_object_unref (tcp_stream);
 		return FALSE;
 	}
@@ -159,8 +167,13 @@ connect_to_server (CamelService *service,
 	if (camel_url_get_param (service->url, "disable_extensions"))
 		flags |= CAMEL_POP3_ENGINE_DISABLE_EXTENSIONS;
 
-	if ((delete_days = (gchar *) camel_url_get_param(service->url,"delete_after")))
-		store->delete_after =  atoi(delete_days);
+	store->keep_on_server = camel_url_get_param (service->url, "keep_on_server") != NULL;
+	store->delete_expunged = camel_url_get_param (service->url, "delete_expunged") != NULL;
+
+	if ((param = camel_url_get_param (service->url, "delete_after")))
+		store->delete_after = atoi (param);
+	else
+		store->delete_after = 0;
 
 	if (!(store->engine = camel_pop3_engine_new (tcp_stream, flags))) {
 		g_set_error (
@@ -252,6 +265,7 @@ connect_to_server (CamelService *service,
 
 static gboolean
 connect_to_server_wrapper (CamelService *service,
+                           GCancellable *cancellable,
                            GError **error)
 {
 	const gchar *ssl_mode;
@@ -278,12 +292,15 @@ connect_to_server_wrapper (CamelService *service,
 		fallback_port = 0;
 	}
 
-	return connect_to_server (service, service->url->host, serv, fallback_port, mode, error);
+	return connect_to_server (
+		service, service->url->host, serv,
+		fallback_port, mode, cancellable, error);
 }
 
 static gint
 try_sasl (CamelPOP3Store *store,
           const gchar *mech,
+          GCancellable *cancellable,
           GError **error)
 {
 	CamelPOP3Stream *stream = store->engine->stream;
@@ -307,7 +324,7 @@ try_sasl (CamelPOP3Store *store,
 		goto ioerror;
 
 	while (1) {
-		if (camel_pop3_stream_line(stream, &line, &len) == -1)
+		if (camel_pop3_stream_line (stream, &line, &len) == -1)
 			goto ioerror;
 		if (strncmp((gchar *) line, "+OK", 3) == 0)
 			break;
@@ -330,10 +347,10 @@ try_sasl (CamelPOP3Store *store,
 		/* If we dont get continuation, or the sasl object's run out of work, or we dont get a challenge,
 		   its a protocol error, so fail, and try reset the server */
 		if (strncmp((gchar *) line, "+ ", 2) != 0
-		    || camel_sasl_get_authenticated(sasl)
-		    || (resp = (guchar *) camel_sasl_challenge_base64(sasl, (const gchar *) line+2, error)) == NULL) {
+		    || camel_sasl_get_authenticated (sasl)
+		    || (resp = (guchar *) camel_sasl_challenge_base64_sync (sasl, (const gchar *) line+2, cancellable, NULL)) == NULL) {
 			camel_stream_printf((CamelStream *)stream, "*\r\n");
-			camel_pop3_stream_line(stream, &line, &len);
+			camel_pop3_stream_line (stream, &line, &len);
 			g_set_error (
 				error, CAMEL_SERVICE_ERROR,
 				CAMEL_SERVICE_ERROR_CANT_AUTHENTICATE,
@@ -344,7 +361,7 @@ try_sasl (CamelPOP3Store *store,
 		}
 
 		ret = camel_stream_printf((CamelStream *)stream, "%s\r\n", resp);
-		g_free(resp);
+		g_free (resp);
 		if (ret == -1)
 			goto ioerror;
 
@@ -366,6 +383,7 @@ static gint
 pop3_try_authenticate (CamelService *service,
                        gboolean reprompt,
                        const gchar *errmsg,
+                       GCancellable *cancellable,
                        GError **error)
 {
 	CamelPOP3Store *store = (CamelPOP3Store *)service;
@@ -412,7 +430,7 @@ pop3_try_authenticate (CamelService *service,
 		d = store->engine->apop;
 
 		while (*d != '\0') {
-			if (!isascii((gint)*d)) {
+			if (!isascii ((gint)*d)) {
 
 				/* README for Translators: The string APOP should not be translated */
 				g_set_error (
@@ -428,7 +446,7 @@ pop3_try_authenticate (CamelService *service,
 			d++;
 		}
 
-		secret = g_alloca(strlen(store->engine->apop)+strlen(service->url->passwd)+1);
+		secret = g_alloca (strlen (store->engine->apop)+strlen (service->url->passwd)+1);
 		sprintf(secret, "%s%s",  store->engine->apop, service->url->passwd);
 		md5asc = g_compute_checksum_for_string (G_CHECKSUM_MD5, secret, -1);
 		pcp = camel_pop3_engine_command_new(store->engine, 0, NULL, NULL, "APOP %s %s\r\n",
@@ -441,8 +459,10 @@ pop3_try_authenticate (CamelService *service,
 		l = store->engine->auth;
 		while (l) {
 			auth = l->data;
-			if (strcmp(auth->authproto, service->url->authmech) == 0)
-				return try_sasl (store, service->url->authmech, error);
+			if (strcmp (auth->authproto, service->url->authmech) == 0)
+				return try_sasl (
+					store, service->url->authmech,
+					cancellable, error);
 			l = l->next;
 		}
 
@@ -452,10 +472,11 @@ pop3_try_authenticate (CamelService *service,
 			_("Unable to connect to POP server %s: "
 			  "No support for requested authentication mechanism."),
 			CAMEL_SERVICE (store)->url->host);
+
 		return 0;
 	}
 
-	while ((status = camel_pop3_engine_iterate(store->engine, pcp)) > 0)
+	while ((status = camel_pop3_engine_iterate (store->engine, pcp)) > 0)
 		;
 
 	if (status == -1) {
@@ -504,7 +525,7 @@ pop3_try_authenticate (CamelService *service,
 	camel_pop3_engine_command_free (store->engine, pcp);
 
 	if (pcu)
-		camel_pop3_engine_command_free(store->engine, pcu);
+		camel_pop3_engine_command_free (store->engine, pcu);
 
 	return status;
 }
@@ -517,7 +538,7 @@ pop3_store_finalize (GObject *object)
 	/* force disconnect so we dont have it run later, after we've cleaned up some stuff */
 	/* SIGH */
 
-	camel_service_disconnect((CamelService *)pop3_store, TRUE, NULL);
+	camel_service_disconnect_sync ((CamelService *)pop3_store, TRUE, NULL);
 
 	if (pop3_store->engine)
 		g_object_unref (pop3_store->engine);
@@ -528,9 +549,25 @@ pop3_store_finalize (GObject *object)
 	G_OBJECT_CLASS (camel_pop3_store_parent_class)->finalize (object);
 }
 
+static gchar *
+pop3_store_get_name (CamelService *service,
+                     gboolean brief)
+{
+	if (brief)
+		return g_strdup_printf (
+			_("POP3 server %s"),
+			service->url->host);
+	else
+		return g_strdup_printf (
+			_("POP3 server for %s on %s"),
+			service->url->user,
+			service->url->host);
+}
+
 static gboolean
-pop3_store_connect (CamelService *service,
-                    GError **error)
+pop3_store_connect_sync (CamelService *service,
+                         GCancellable *cancellable,
+                         GError **error)
 {
 	CamelPOP3Store *store = (CamelPOP3Store *)service;
 	gboolean reprompt = FALSE;
@@ -547,21 +584,22 @@ pop3_store_connect (CamelService *service,
 		root = camel_session_get_storage_path (session, service, error);
 		if (root) {
 			store->cache = camel_data_cache_new (root, error);
-			g_free(root);
+			g_free (root);
 			if (store->cache) {
-				/* Ensure cache will never expire, otherwise it causes redownload of messages */
-				camel_data_cache_set_expire_age (store->cache, -1);
-				camel_data_cache_set_expire_access (store->cache, -1);
+				/* Default cache expiry - 1 week or not visited in a day */
+				camel_data_cache_set_expire_age (store->cache, 60*60*24*7);
+				camel_data_cache_set_expire_access (store->cache, 60*60*24);
 			}
 		}
 	}
 
-	if (!connect_to_server_wrapper (service, error))
+	if (!connect_to_server_wrapper (service, cancellable, error))
 		return FALSE;
 
 	while (1) {
 		status = pop3_try_authenticate (
-			service, reprompt, errbuf, &local_error);
+			service, reprompt, errbuf,
+			cancellable, &local_error);
 		g_free (errbuf);
 		errbuf = NULL;
 
@@ -585,7 +623,7 @@ pop3_store_connect (CamelService *service,
 
 	if (local_error != NULL) {
 		g_propagate_error (error, local_error);
-		camel_service_disconnect(service, TRUE, NULL);
+		camel_service_disconnect_sync (service, TRUE, NULL);
 		return FALSE;
 	}
 
@@ -598,9 +636,10 @@ pop3_store_connect (CamelService *service,
 }
 
 static gboolean
-pop3_store_disconnect (CamelService *service,
-                       gboolean clean,
-                       GError **error)
+pop3_store_disconnect_sync (CamelService *service,
+                            gboolean clean,
+                            GCancellable *cancellable,
+                            GError **error)
 {
 	CamelServiceClass *service_class;
 	CamelPOP3Store *store = CAMEL_POP3_STORE (service);
@@ -609,14 +648,14 @@ pop3_store_disconnect (CamelService *service,
 		CamelPOP3Command *pc;
 
 		pc = camel_pop3_engine_command_new(store->engine, 0, NULL, NULL, "QUIT\r\n");
-		while (camel_pop3_engine_iterate(store->engine, NULL) > 0)
+		while (camel_pop3_engine_iterate (store->engine, NULL) > 0)
 			;
-		camel_pop3_engine_command_free(store->engine, pc);
+		camel_pop3_engine_command_free (store->engine, pc);
 	}
 
 	/* Chain up to parent's disconnect() method. */
 	service_class = CAMEL_SERVICE_CLASS (camel_pop3_store_parent_class);
-	if (!service_class->disconnect (service, clean, error))
+	if (!service_class->disconnect_sync (service, clean, cancellable, error))
 		return FALSE;
 
 	g_object_unref (store->engine);
@@ -626,8 +665,9 @@ pop3_store_disconnect (CamelService *service,
 }
 
 static GList *
-pop3_store_query_auth_types (CamelService *service,
-                             GError **error)
+pop3_store_query_auth_types_sync (CamelService *service,
+                                  GCancellable *cancellable,
+                                  GError **error)
 {
 	CamelServiceClass *service_class;
 	CamelPOP3Store *store = CAMEL_POP3_STORE (service);
@@ -636,16 +676,17 @@ pop3_store_query_auth_types (CamelService *service,
 
 	/* Chain up to parent's query_auth_types() method. */
 	service_class = CAMEL_SERVICE_CLASS (camel_pop3_store_parent_class);
-	types = service_class->query_auth_types (service, &local_error);
+	types = service_class->query_auth_types_sync (
+		service, cancellable, &local_error);
 
 	if (local_error != NULL) {
 		g_propagate_error (error, local_error);
 		return NULL;
 	}
 
-	if (connect_to_server_wrapper (service, NULL)) {
-		types = g_list_concat(types, g_list_copy(store->engine->auth));
-		pop3_store_disconnect (service, TRUE, NULL);
+	if (connect_to_server_wrapper (service, cancellable, NULL)) {
+		types = g_list_concat (types, g_list_copy (store->engine->auth));
+		pop3_store_disconnect_sync (service, TRUE, cancellable, NULL);
 	} else {
 		g_set_error (
 			error, CAMEL_SERVICE_ERROR,
@@ -657,26 +698,21 @@ pop3_store_query_auth_types (CamelService *service,
 	return types;
 }
 
-static gchar *
-pop3_store_get_name (CamelService *service,
-                     gboolean brief)
+static gboolean
+pop3_store_can_refresh_folder (CamelStore *store,
+                               CamelFolderInfo *info,
+                               GError **error)
 {
-	if (brief)
-		return g_strdup_printf (
-			_("POP3 server %s"),
-			service->url->host);
-	else
-		return g_strdup_printf (
-			_("POP3 server for %s on %s"),
-			service->url->user,
-			service->url->host);
+	/* any pop3 folder can be refreshed */
+	return TRUE;
 }
 
 static CamelFolder *
-pop3_store_get_folder (CamelStore *store,
-                       const gchar *folder_name,
-                       guint32 flags,
-                       GError **error)
+pop3_store_get_folder_sync (CamelStore *store,
+                            const gchar *folder_name,
+                            CamelStoreGetFolderFlags flags,
+                            GCancellable *cancellable,
+                            GError **error)
 {
 	if (g_ascii_strcasecmp (folder_name, "inbox") != 0) {
 		g_set_error (
@@ -686,22 +722,15 @@ pop3_store_get_folder (CamelStore *store,
 		return NULL;
 	}
 
-	return camel_pop3_folder_new (store, error);
-}
-
-static CamelFolder *
-pop3_store_get_trash (CamelStore *store,
-                      GError **error)
-{
-	/* no-op */
-	return NULL;
+	return camel_pop3_folder_new (store, cancellable, error);
 }
 
 static CamelFolderInfo *
-pop3_store_get_folder_info (CamelStore *store,
-                            const gchar *top,
-                            guint32 flags,
-                            GError **error)
+pop3_store_get_folder_info_sync (CamelStore *store,
+                                 const gchar *top,
+                                 CamelStoreGetFolderInfoFlags flags,
+                                 GCancellable *cancellable,
+                                 GError **error)
 {
 	g_set_error (
 		error, CAMEL_STORE_ERROR,
@@ -711,13 +740,13 @@ pop3_store_get_folder_info (CamelStore *store,
 	return NULL;
 }
 
-static gboolean
-pop3_store_can_refresh_folder (CamelStore *store,
-                               CamelFolderInfo *info,
-                               GError **error)
+static CamelFolder *
+pop3_store_get_trash_folder_sync (CamelStore *store,
+                                  GCancellable *cancellable,
+                                  GError **error)
 {
-	/* any pop3 folder can be refreshed */
-	return TRUE;
+	/* no-op */
+	return NULL;
 }
 
 static void
@@ -731,16 +760,16 @@ camel_pop3_store_class_init (CamelPOP3StoreClass *class)
 	object_class->finalize = pop3_store_finalize;
 
 	service_class = CAMEL_SERVICE_CLASS (class);
-	service_class->connect = pop3_store_connect;
-	service_class->disconnect = pop3_store_disconnect;
-	service_class->query_auth_types = pop3_store_query_auth_types;
 	service_class->get_name = pop3_store_get_name;
+	service_class->connect_sync = pop3_store_connect_sync;
+	service_class->disconnect_sync = pop3_store_disconnect_sync;
+	service_class->query_auth_types_sync = pop3_store_query_auth_types_sync;
 
 	store_class = CAMEL_STORE_CLASS (class);
-	store_class->get_folder = pop3_store_get_folder;
-	store_class->get_trash = pop3_store_get_trash;
-	store_class->get_folder_info = pop3_store_get_folder_info;
 	store_class->can_refresh_folder = pop3_store_can_refresh_folder;
+	store_class->get_folder_sync = pop3_store_get_folder_sync;
+	store_class->get_folder_info_sync = pop3_store_get_folder_info_sync;
+	store_class->get_trash_folder_sync = pop3_store_get_trash_folder_sync;
 }
 
 static void
@@ -766,11 +795,11 @@ camel_pop3_store_expunge (CamelPOP3Store *store,
 	pc = camel_pop3_engine_command_new (
 		store->engine, 0, NULL, NULL, "QUIT\r\n");
 
-	while (camel_pop3_engine_iterate(store->engine, NULL) > 0)
+	while (camel_pop3_engine_iterate (store->engine, NULL) > 0)
 		;
 
-	camel_pop3_engine_command_free(store->engine, pc);
+	camel_pop3_engine_command_free (store->engine, pc);
 
-	camel_service_disconnect (CAMEL_SERVICE (store), FALSE, error);
+	camel_service_disconnect_sync (CAMEL_SERVICE (store), FALSE, error);
 }
 
