@@ -301,7 +301,7 @@ ep_need_proxy_http (EProxy* proxy, const gchar * host)
 		so_addr = soup_address_get_sockaddr (addr, &addr_len);
 
 		/* This will never happen, since we have already called
-		   soup_address_resolve_sync().
+		   soup_address_resolve_sync ().
 		*/
 		if (!so_addr)
 			return TRUE;
@@ -406,7 +406,7 @@ ep_manipulate_ipv4 (ProxyHostAddr *host_addr,
 }
 
 static void
-ipv6_network_addr(const struct in6_addr *addr, const struct in6_addr *mask,
+ipv6_network_addr (const struct in6_addr *addr, const struct in6_addr *mask,
                   struct in6_addr *res)
 {
 	gint i;
@@ -438,7 +438,7 @@ ep_manipulate_ipv6 (ProxyHostAddr *host_addr,
         }
         if (netmask) {
 		gchar *endptr;
-		gint width = strtol(netmask, &endptr, 10);
+		gint width = strtol (netmask, &endptr, 10);
 
 		if (*endptr != '\0' || width < 0 || width > 128) {
 			has_error = TRUE;
@@ -450,7 +450,7 @@ ep_manipulate_ipv6 (ProxyHostAddr *host_addr,
 			mask->s6_addr[i] = 0xff;
 		}
 		mask->s6_addr[i] = (0xff << (8 - width % 8)) & 0xff;
-		ipv6_network_addr(addr, mask, addr);
+		ipv6_network_addr (addr, mask, addr);
         } else {
 		for (i = 0; i < 16; ++i) {
 			mask->s6_addr[i] = 0xff;
@@ -498,7 +498,7 @@ ep_parse_ignore_host (gpointer data, gpointer user_data)
 		so_addr = soup_address_get_sockaddr (addr, &addr_len);
 
 		/* This will never happen, since we have already called
-		   soup_address_resolve_sync().
+		   soup_address_resolve_sync ().
 		*/
 		if (!so_addr)
 			goto error;
@@ -555,6 +555,40 @@ ep_change_uri (SoupURI **soup_uri, const gchar *uri)
 	return changed;
 }
 
+static gchar *
+update_proxy_uri (const gchar *uri, const gchar *proxy_user, const gchar *proxy_pw)
+{
+	gchar *res, *user = NULL, *pw = NULL;
+	gboolean is_https;
+
+	g_return_val_if_fail (uri != NULL, NULL);
+
+	if (proxy_user && *proxy_user) {
+		user = soup_uri_encode (proxy_user, ":/;#@?\\");
+		if (proxy_pw)
+			pw = soup_uri_encode (proxy_pw, ":/;#@?\\");
+	}
+
+	if (!user)
+		return g_strdup (uri);
+
+	/*  here can be only http or https and nothing else */
+	is_https = g_str_has_prefix (uri, "https://");
+
+	res = g_strdup_printf ("%s://%s%s%s%s%s",
+		is_https ? "https" : "http",
+		user ? user : "",
+		pw ? ":" : "",
+		pw ? pw : "",
+		(user || pw) ? "@" : "",
+		uri + strlen ("http://") + (is_https ? 1 : 0));
+
+	g_free (user);
+	g_free (pw);
+
+	return res;
+}
+
 static void
 ep_set_proxy (GConfClient *client,
 	      gpointer user_data,
@@ -627,22 +661,14 @@ ep_set_proxy (GConfClient *client,
 		proxy_user = gconf_client_get_string (client, RIGHT_KEY (HTTP_AUTH_USER), NULL);
 		proxy_pw = gconf_client_get_string (client, RIGHT_KEY (HTTP_AUTH_PWD), NULL);
 
-		if (uri_http && proxy_user && *proxy_user && proxy_pw && *proxy_pw) {
+		if (uri_http && proxy_user && *proxy_user) {
 			tmp = uri_http;
-			uri_http = g_strdup_printf ("http://%s:%s@%s", proxy_user, proxy_pw, tmp + strlen ("http://"));
-		} else if (uri_http && proxy_user && *proxy_user) {
-			/* proxy without password, just try it */
-			tmp = uri_http;
-			uri_http = g_strdup_printf ("http://%s@%s", proxy_user, tmp + strlen ("http://"));
+			uri_http = update_proxy_uri (uri_http, proxy_user, proxy_pw);
 		}
 
-		if (uri_https && proxy_user && *proxy_user && proxy_pw && *proxy_pw) {
+		if (uri_https && proxy_user && *proxy_user) {
 			tmps = uri_https;
-			uri_https = g_strdup_printf ("https://%s:%s@%s", proxy_user, proxy_pw, tmps + strlen ("https://"));
-		} else if (uri_https && proxy_user && *proxy_user) {
-			/* proxy without password, just try it */
-			tmps = uri_https;
-			uri_https = g_strdup_printf ("https://%s@%s", proxy_user, tmps + strlen ("https://"));
+			uri_https = update_proxy_uri (uri_https, proxy_user, proxy_pw);
 		}
 
 		g_free (proxy_user);
