@@ -4,7 +4,7 @@
  * Sivaiah Nallagatla <snallagatla@novell.com>
  *
  *
- * Copyright 2003, Novell, Inc.
+ * Copyright (C) 1999-2008 Novell, Inc. (www.novell.com)
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of version 2 of the GNU Lesser General Public
@@ -24,9 +24,8 @@
 #ifndef E_GW_FILTER_H
 #define E_GW_FILTER_H
 
-
-#include <libsoup/soup-soap-message.h>
-#include <libsoup/soup-soap-response.h>
+#include "soup-soap-message.h"
+#include "soup-soap-response.h"
 
 G_BEGIN_DECLS
 
@@ -40,7 +39,6 @@ typedef struct _EGwFilter        EGwFilter;
 typedef struct _EGwFilterClass   EGwFilterClass;
 typedef struct _EGwFilterPrivate EGwFilterPrivate;
 
-
 typedef enum {
 	E_GW_FILTER_OP_AND,
 	E_GW_FILTER_OP_OR,
@@ -69,10 +67,10 @@ struct _EGwFilterClass {
 };
 
 GType       e_gw_filter_get_type (void);
-EGwFilter*  e_gw_filter_new(void);
-void        e_gw_filter_add_filter_component (EGwFilter *filter, EGwFilterOpType operation, const char *field_name, const char *field_value);
+EGwFilter*  e_gw_filter_new (void);
+void        e_gw_filter_add_filter_component (EGwFilter *filter, EGwFilterOpType operation, const gchar *field_name, const gchar *field_value);
 void        e_gw_filter_append_to_soap_message (EGwFilter *filter, SoupSoapMessage *msg);
-void        e_gw_filter_group_conditions (EGwFilter *filter, EGwFilterOpType operation, int num_of_condtions);
+void        e_gw_filter_group_conditions (EGwFilter *filter, EGwFilterOpType operation, gint num_of_condtions);
 
 G_END_DECLS
 
